@@ -1,21 +1,21 @@
 
 /***************************************************************************
- * FingerPrintResults -- The FingerPrintResults class the results of OS    *
+ * FingerPrintResults.h -- The FingerPrintResults class the results of OS  *
  * fingerprint matching against a certain host.                            *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,7 +24,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -50,16 +50,16 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -77,7 +77,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -91,13 +91,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: FingerPrintResults.h 2396 2004-08-29 09:12:05Z fyodor $ */
+/* $Id: FingerPrintResults.h 7640 2008-05-22 20:45:32Z fyodor $ */
 
 #ifndef FINGERPRINTRESULTS_H
 #define FINGERPRINTRESULTS_H
@@ -105,6 +105,7 @@
 class FingerPrintResults;
 
 #include "nmap.h"
+#include "global_structures.h"
 
 /* Maximum number of results allowed in one of these things ... */
 #define MAX_FP_RESULTS 36
@@ -131,23 +132,39 @@ class FingerPrintResults {
   int overall_results; /* OSSCAN_TOOMANYMATCHES, OSSCAN_NOMATCHES, 
 			  OSSCAN_SUCCESS, etc */
 
-  /* Ensures that the results are available and then returns them.  You should only call
-   this AFTER all matching has been completed (because results are cached and won't change
-   if new prints[] are added.)  All OS Classes in the results will be unique, and if there are 
-   any perfect (accuracy 1.0) matches, only those will be returned */
+  /* Ensures that the results are available and then returns them.
+   You should only call this AFTER all matching has been completed
+   (because results are cached and won't change if new prints[] are
+   added.)  All OS Classes in the results will be unique, and if there
+   are any perfect (accuracy 1.0) matches, only those will be
+   returned */
   const struct OS_Classification_Results *getOSClassification();
 
-  int osscan_opentcpport; /* Open port used for scannig (if one found -- 
+  int osscan_opentcpport; /* Open TCP port used for scannig (if one found -- 
 			  otherwise -1) */
-  int osscan_closedtcpport; /* Closed port used for scannig (if one found -- 
+  int osscan_closedtcpport; /* Closed TCP port used for scannig (if one found -- 
 			    otherwise -1) */
-  FingerPrint *FPs[10]; /* Fingerprint data obtained from host */
+  int osscan_closedudpport;  /* Closed UDP port used for scannig (if one found -- 
+			    otherwise -1) */
+  int distance; /* How "far" is this FP gotten from? */
+  int distance_guess; /* How "far" is this FP gotten from? by guessing based on ttl. */
+
+  /* The largest ratio we have seen of time taken vs. target time
+     between sending 1st tseq probe and sending first ICMP echo probe.
+     Zero means we didn't see any ratios (the tseq probes weren't
+     sent), 1 is ideal, and larger values are undesirable from a
+     consistancy standpoint. */
+  double maxTimingRatio;
+
+  FingerPrint **FPs; /* Fingerprint data obtained from host */
   int numFPs;
   int goodFP;
 
-  /* Are the attributes of this fingerprint good enough to warrant submission to the official DB? */
-  bool fingerprintSuitableForSubmission(); 
-                                          
+/* If the fingerprint is of potentially poor quality, we don't want to
+   print it and ask the user to submit it.  In that case, the reason
+   for skipping the FP is returned as a static string.  If the FP is
+   great and should be printed, NULL is returned. */
+  const char *OmitSubmissionFP();
 
  private:
   bool isClassified; // Whether populateClassification() has been called
