@@ -24,7 +24,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -59,7 +59,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -91,13 +91,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: NmapOps.cc 9347 2008-08-05 19:28:51Z david $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -116,10 +116,6 @@ NmapOps::NmapOps() {
 }
 
 NmapOps::~NmapOps() {
-  if (ping_synprobes) free(ping_synprobes);
-  if (ping_ackprobes) free(ping_ackprobes);
-  if (ping_udpprobes) free(ping_udpprobes);
-  if (ping_protoprobes) free(ping_protoprobes);
   if (datadir) free(datadir);
   if (xsl_stylesheet) free(xsl_stylesheet);
 }
@@ -195,6 +191,8 @@ void NmapOps::Initialize() {
 #endif
   debugging = 0;
   verbose = 0;
+  min_packet_send_rate = 0.0; /* Unset. */
+  max_packet_send_rate = 0.0; /* Unset. */
   randomize_hosts = 0;
   sendpref = PACKET_SEND_NOPREF;
   spoofsource = 0;
@@ -202,12 +200,11 @@ void NmapOps::Initialize() {
   device[0] = '\0';
   interactivemode = 0;
   ping_group_sz = PING_GROUP_SZ;
+  nogcc = 0;
   generate_random_ips = 0;
   reference_FPs = NULL;
   magic_port = 33000 + (get_random_uint() % 31000);
   magic_port_set = 0;
-  num_ping_synprobes = num_ping_ackprobes = num_ping_udpprobes = num_ping_protoprobes = 0;
-  ping_synprobes = ping_ackprobes = ping_udpprobes = ping_protoprobes = NULL;
   timing_level = 3;
   max_parallelism = 0;
   min_parallelism = 0;
@@ -276,6 +273,8 @@ void NmapOps::Initialize() {
   scripttrace = 0;
   scriptupdatedb = 0;
 #endif
+  memset(&sourcesock, 0, sizeof(sourcesock));
+  sourcesocklen = 0;
 }
 
 bool NmapOps::TCPScan() {
@@ -306,12 +305,7 @@ void NmapOps::ValidateOptions() {
 #else
 	const char *privreq = "root privileges";
 #endif
-  if (pingtype == PINGTYPE_UNKNOWN) {
-    if (isr00t && af() == AF_INET) pingtype = DEFAULT_PING_TYPES;
-    else pingtype = PINGTYPE_TCP; // if nonr00t or IPv6
-    getpts_simple(DEFAULT_TCP_PROBE_PORT_SPEC, SCAN_TCP_PORT, &o.ping_ackprobes, &o.num_ping_ackprobes);
-    assert(o.num_ping_ackprobes > 0);
-  }
+
 
   /* Insure that at least one scantype is selected */
   if (TCPScan() + UDPScan() + ipprotscan + listscan + pingscan == 0) {
@@ -321,23 +315,6 @@ void NmapOps::ValidateOptions() {
     //    if (verbose) error("No tcp, udp, or ICMP scantype specified, assuming %s scan. Use -sP if you really don't want to portscan (and just want to see what hosts are up).", synscan? "SYN Stealth" : "vanilla tcp connect()");
   }
 
-  if ((pingtype & PINGTYPE_TCP) && (!isr00t || af() != AF_INET)) {
-    /* We will have to do a connect() style ping */
-    if (num_ping_synprobes && num_ping_ackprobes) {
-      fatal("Cannot use both SYN and ACK ping probes if you are nonroot or using IPv6");
-    }
-    
-    /* Pretend we wanted SYN probes all along. */
-    if (num_ping_ackprobes > 0) { 
-      num_ping_synprobes = num_ping_ackprobes;
-      ping_synprobes = ping_ackprobes;
-      num_ping_ackprobes = 0;
-      ping_ackprobes = NULL;
-    }
-    pingtype &= ~PINGTYPE_TCP_USE_ACK;
-    pingtype |= PINGTYPE_TCP_USE_SYN;
-  }
-
   if (pingtype != PINGTYPE_NONE && spoofsource) {
     error("WARNING:  If -S is being used to fake your source address, you may also have to use -e <interface> and -PN .  If you are using it to specify your real source address, you can ignore this warning.");
   }
@@ -375,6 +352,10 @@ void NmapOps::ValidateOptions() {
    fatal("Ping scan is not valid with any other scan types (the other ones all include a ping scan");
  }
 
+ if (af() == AF_INET6 && (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS))) {
+   fatal("ICMP Echo, Timestamp and Address Mask pings are only valid for IPv4.");
+ }
+
  if (sendpref == PACKET_SEND_NOPREF) {
 #ifdef WIN32
    sendpref = PACKET_SEND_ETH_STRONG;
@@ -384,18 +365,6 @@ void NmapOps::ValidateOptions() {
  }
 /* We start with stuff users should not do if they are not root */
   if (!isr00t) {
-
-#ifndef WIN32	/*	Win32 has perfectly fine ICMP socket support */
-    if (pingtype & (PINGTYPE_ICMP_PING|PINGTYPE_ICMP_MASK|PINGTYPE_ICMP_TS)) {
-      error("Warning:  You are not root -- using TCP pingscan rather than ICMP");
-      pingtype = PINGTYPE_TCP;
-      if (num_ping_synprobes == 0)
-	{
-          getpts_simple(DEFAULT_TCP_PROBE_PORT_SPEC, SCAN_TCP_PORT, &o.ping_synprobes, &o.num_ping_synprobes);
-          assert(o.num_ping_synprobes > 0);
-	}
-    }
-#endif
     
     if (ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan) {
       fatal("You requested a scan type which requires %s.", privreq);
@@ -412,9 +381,6 @@ void NmapOps::ValidateOptions() {
     if (osscan) {
       fatal("TCP/IP fingerprinting (for OS scan) requires %s.", privreq);
     }
-
-    if (ipoptionslen)
-      fatal("Sorry, using ip options requires %s.", privreq);
   }
   
   
@@ -478,6 +444,10 @@ void NmapOps::ValidateOptions() {
   if (max_parallelism && min_parallelism && (min_parallelism > max_parallelism)) {
     fatal("--min-parallelism=%i must be less than or equal to --max-parallelism=%i",min_parallelism,max_parallelism);
   }
+
+  if (min_packet_send_rate != 0.0 && max_packet_send_rate != 0.0 && min_packet_send_rate > max_packet_send_rate) {
+    fatal("--min-rate=%g must be less than or equal to --max-rate=%g", min_packet_send_rate, max_packet_send_rate);
+  }
   
   if (af() == AF_INET6 && (numdecoys|osscan|bouncescan|fragscan|ackscan|finscan|idlescan|ipprotscan|maimonscan|nullscan|synscan|udpscan|windowscan|xmasscan)) {
     fatal("Sorry -- IPv6 support is currently only available for connect() scan (-sT), ping scan (-sP), and list scan (-sL).  OS detection and decoys are also not supported with IPv6.  Further support is under consideration.");
@@ -491,7 +461,12 @@ void NmapOps::ValidateOptions() {
 
   if(ipoptions && osscan)
     error("WARNING: Ip options are NOT used while OS scanning!");
-    
+
+#ifndef NOLUA
+  /* Make sure nmap.registry.args is available (even if it's empty) */
+  if (!scriptargs)
+    scriptargs = strdup("");
+#endif
 }
 
 void NmapOps::setMaxOSTries(int mot) {
@@ -550,7 +525,7 @@ void NmapOps::setMaxHostGroupSz(unsigned int sz) {
      If this is never called, a default stylesheet distributed with
      Nmap is used.  If you call it with NULL as the xslname, no
      stylesheet line is printed. */
-void NmapOps::setXSLStyleSheet(char *xslname) {
+void NmapOps::setXSLStyleSheet(const char *xslname) {
   if (xsl_stylesheet) free(xsl_stylesheet);
   xsl_stylesheet = xslname? strdup(xslname) : NULL;
 }
