@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -24,7 +24,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -59,7 +59,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -77,7 +77,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -91,18 +91,18 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOutputTable.h 3869 2006-08-25 01:47:49Z fyodor $ */
+/* $Id: NmapOutputTable.h 7640 2008-05-22 20:45:32Z fyodor $ */
 
 #ifndef NMAPOUTPUTTABLE_H
 #define NMAPOUTPUTTABLE_H
 
-#include <assert.h>
+#include "nmap.h"
 
 #ifndef __attribute__
 #define __attribute__(args)
@@ -118,11 +118,15 @@ struct NmapOutputTableCell {
   char *str;
   int strlength;
   bool weAllocated; // If we allocated str, we must free it.
+  bool fullrow;
 };
 
 class NmapOutputTable {
  public:
-  // Create a table of the given dimensions
+  // Create a table of the given dimensions. Any completely
+  // blank rows will be removed when printableTable() is called.
+  // If the number of table rows is unknown then the highest
+  // number of possible rows should be specified.
   NmapOutputTable(int nrows, int ncols);
   ~NmapOutputTable();
 
@@ -130,9 +134,14 @@ class NmapOutputTable {
   // ptr (and you better not free it until this table is destroyed ).  Skip the itemlen parameter if you
   // don't know (and the function will use strlen).
   void addItem(unsigned int row, unsigned int column, bool copy, const char *item, int itemlen = -1);
+  // Same as above but if fullrow is true, 'item' spans across all columns. The spanning starts from
+  // the column argument (ie. 0 will be the first column)
+  void addItem(unsigned int row, unsigned int column, bool fullrow, bool copy, const char *item, int itemlen = -1);
+
   // Like addItem except this version takes a printf-style format string followed by varargs
-  void addItemFormatted(unsigned int row, unsigned int column, const char *fmt, ...)
-    __attribute__ ((format (printf, 4, 5)));
+  void addItemFormatted(unsigned int row, unsigned int column, bool fullrow, const char *fmt, ...)
+	  __attribute__ ((format (printf, 4, 5)));
+
   // Returns the maximum size neccessary to create a printableTable() (the 
   // actual size could be less);
   int printableSize();
@@ -142,10 +151,12 @@ class NmapOutputTable {
   // function again, and it will also be invalidated if you free the
   // table. If size is not NULL, it will be filled with the size of
   // the ASCII table in bytes (not including the terminating NUL)
+  // All blank rows will be removed from the returned string
   char *printableTable(int *size);
 
  private:
 
+  bool emptyRow(unsigned int nrow);
   // The table, squished into 1D.  Access a member via getCellAddy
   struct NmapOutputTableCell *table;
   struct NmapOutputTableCell *getCellAddy(unsigned int row, unsigned int col) {
@@ -168,7 +179,3 @@ class NmapOutputTable {
 
 #endif /* NMAPOUTPUTTABLE_H */
 
-
-
-
-
