@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -60,7 +60,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -92,13 +92,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: Target.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: Target.cc 8759 2008-07-11 06:12:38Z david $ */
 
 #ifdef WIN32
 #include "nmap_winconfig.h"
@@ -122,7 +122,6 @@ void Target::Initialize() {
   targetname = NULL;
   memset(&seq, 0, sizeof(seq));
   distance = -1;
-  FPR1 = NULL;
   FPR = NULL;
   osscan_flag = OS_NOTPERF;
   wierd_responses = flags = 0;
@@ -140,10 +139,13 @@ void Target::Initialize() {
   MACaddress_set = SrcMACaddress_set = NextHopMACaddress_set = false;
   htn.msecs_used = 0;
   htn.toclock_running = false;
+  htn.host_start = htn.host_end = 0;
   interface_type = devt_other;
 	devname[0] = '\0';
 	devfullname[0] = '\0';
   state_reason_init(&reason);
+  memset(&pingprobe, 0, sizeof(pingprobe));
+  pingprobe_state = PORT_UNKNOWN;
 }
 
 
@@ -165,7 +167,6 @@ Target::~Target() {
 }
 
 void Target::FreeInternal() {
-
   /* Free the DNS name if we resolved one */
   if (hostname)
     free(hostname);
@@ -178,7 +179,6 @@ void Target::FreeInternal() {
     nameIPBuf = NULL;
   }
 
-  if (FPR1) delete FPR1;
   if (FPR) delete FPR;
 }
 
@@ -391,6 +391,7 @@ void Target::startTimeOutClock(const struct timeval *now) {
   htn.toclock_running = true;
   if (now) htn.toclock_start = *now;
   else gettimeofday(&htn.toclock_start, NULL);
+  if (!htn.host_start) htn.host_start = htn.toclock_start.tv_sec;
 }
   /* The complement to startTimeOutClock. */
 void Target::stopTimeOutClock(const struct timeval *now) {
@@ -400,6 +401,7 @@ void Target::stopTimeOutClock(const struct timeval *now) {
   if (now) tv = *now;
   else gettimeofday(&tv, NULL);
   htn.msecs_used += TIMEVAL_MSEC_SUBTRACT(tv, htn.toclock_start);
+  htn.host_end = tv.tv_sec;
 }
   /* Returns whether the host is timedout.  If the timeoutclock is
      running, counts elapsed time for that.  Pass NULL if you don't have the
@@ -475,3 +477,4 @@ void Target::osscanSetFlag(int flag) {
 		osscan_flag = flag;
 }
 
+
