@@ -26,7 +26,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -61,7 +61,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -93,13 +93,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: TargetGroup.cc 9934 2008-09-04 14:27:55Z david $ */
 
 #include "TargetGroup.h"
 #include "NmapOps.h"
@@ -129,7 +129,7 @@ int  TargetGroup::rewind() {
   if (targets_type == IPV4_NETMASK) {
       currentaddr = startaddr;
       if (startaddr.s_addr <= endaddr.s_addr) { 
-	ipsleft = endaddr.s_addr - startaddr.s_addr + 1;
+	ipsleft = ((unsigned long long) (endaddr.s_addr - startaddr.s_addr)) + 1;
 	return 0; 
       }
       else
@@ -139,8 +139,10 @@ int  TargetGroup::rewind() {
    * the ips by the number of values in the columns */
   else if (targets_type == IPV4_RANGES) {
     memset((char *)current, 0, sizeof(current));
-    ipsleft = (last[0] + 1) * (last[1] + 1) *
-      (last[2] + 1) * (last[3] + 1);
+    ipsleft = (unsigned long long) (last[0] + 1) *
+              (unsigned long long) (last[1] + 1) *
+              (unsigned long long) (last[2] + 1) *
+              (unsigned long long) (last[3] + 1);
     return 0;
   }
 #if HAVE_IPV6
@@ -167,7 +169,6 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
   char *addy[5];
   char *hostexp = strdup(target_expr);
   struct hostent *target;
-  unsigned long longtmp;
   namedhost = 0;
 
   if (targets_type != TYPE_NONE)
@@ -188,8 +189,8 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     target_net = strtok(hostexp, "/");
     s = strtok(NULL, "");    /* find the end of the token from hostexp */
     netmask  = ( s ) ? atoi(s) : 32;
-    if ((int) netmask <= 0 || netmask > 32) {
-      error("Illegal netmask value (%d), must be /1 - /32 .  Assuming /32 (one host)", netmask);
+    if ((int) netmask < 0 || netmask > 32) {
+      error("Illegal netmask value (%d), must be /0 - /32 .  Assuming /32 (one host)", netmask);
       netmask = 32;
     }
     for(i=0; *(hostexp + i); i++) 
@@ -215,12 +216,20 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
 	  return 1;
 	}
       } 
-      longtmp = ntohl(startaddr.s_addr);
-      startaddr.s_addr = longtmp & (unsigned long) (0 - (1<<(32 - netmask)));
-      endaddr.s_addr = longtmp | (unsigned long)  ((1<<(32 - netmask)) - 1);
+      if (netmask) {
+        unsigned long longtmp = ntohl(startaddr.s_addr);
+        startaddr.s_addr = longtmp & (unsigned long) (0 - (1<<(32 - netmask)));
+        endaddr.s_addr = longtmp | (unsigned long)  ((1<<(32 - netmask)) - 1);
+      } else {
+        /* The above calculations don't work for a /0 netmask, though at first
+         * glance it appears that they would
+         */
+        startaddr.s_addr = 0;
+        endaddr.s_addr = 0xffffffff;
+      }
       currentaddr = startaddr;
       if (startaddr.s_addr <= endaddr.s_addr) { 
-	ipsleft = endaddr.s_addr - startaddr.s_addr + 1;
+	ipsleft = ((unsigned long long) (endaddr.s_addr - startaddr.s_addr)) + 1;
 	free(hostexp); 
 	return 0; 
       }
@@ -272,8 +281,10 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
       }
     }
   memset((char *)current, 0, sizeof(current));
-  ipsleft = (last[0] + 1) * (last[1] + 1) *
-    (last[2] + 1) * (last[3] + 1);
+  ipsleft = (unsigned long long) (last[0] + 1) *
+            (unsigned long long) (last[1] + 1) *
+            (unsigned long long) (last[2] + 1) *
+            (unsigned long long) (last[3] + 1);
   }
   else {
 #if HAVE_IPV6
@@ -288,7 +299,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     memset(&hints, 0, sizeof(hints));
     hints.ai_family = PF_INET6;
     rc = getaddrinfo(hostexp, NULL, &hints, &result);
-    if (rc != 0) {
+    if (rc != 0 || result == NULL) {
       error("Failed to resolve given IPv6 hostname/IP: %s.  Note that you can't use '/mask' or '[1-4,7,100-]' style ranges for IPv6.  Error code %d: %s", hostexp, rc, gai_strerror(rc));
       free(hostexp);
       if (result) freeaddrinfo(result);
@@ -296,7 +307,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     }
     assert(result->ai_addrlen == sizeof(struct sockaddr_in6));
     struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *) result->ai_addr;
-    memcpy(ip6.s6_addr, sin6->sin6_addr.s6_addr, 16);
+    memcpy(&ip6, sin6, sizeof(struct sockaddr_in6));
     ipsleft = 1;
     freeaddrinfo(result);
 #else // HAVE_IPV6
@@ -443,7 +454,8 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
 #ifdef SIN_LEN
     sin6->sin6_len = *sslen;
 #endif /* SIN_LEN */
-    memcpy(sin6->sin6_addr.s6_addr, ip6.s6_addr, 16);
+    memcpy(sin6->sin6_addr.s6_addr, ip6.sin6_addr.s6_addr, 16);
+    sin6->sin6_scope_id = ip6.sin6_scope_id;
 #else
     fatal("IPV6 not supported on this platform");
 #endif // HAVE_IPV6
