@@ -284,7 +284,9 @@ intf_set(intf_t *intf, const struct intf_entry *entry)
 	/* Set interface MTU. */
 	if (entry->intf_mtu != 0) {
 		ifr.ifr_mtu = entry->intf_mtu;
+#ifdef SIOCSIFMTU
 		if (ioctl(intf->fd, SIOCSIFMTU, &ifr) < 0)
+#endif
 			return (-1);
 	}
 	/* Set interface address. */
@@ -396,7 +398,9 @@ _intf_get_noalias(intf_t *intf, struct intf_entry *entry)
 	_intf_set_type(entry);
 	
 	/* Get interface MTU. */
+#ifdef SIOCGIFMTU
 	if (ioctl(intf->fd, SIOCGIFMTU, &ifr) < 0)
+#endif
 		return (-1);
 	entry->intf_mtu = ifr.ifr_mtu;
 
