@@ -5,11 +5,11 @@
  * using libnbase can guarantee the availability of functions like         *
  * (v)snprintf and inet_pton.  This library also provides consistency and  *
  * extended features for some functions.  It was originally written for    *
- * use in the Nmap Security Scanner ( http://www.insecure.org/nmap/ ).     *
+ * use in the Nmap Security Scanner ( http://nmap.org ).                   *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -28,7 +28,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -42,7 +42,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -63,7 +63,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -81,7 +81,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -95,13 +95,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h 3899 2006-08-29 05:42:35Z fyodor $ */
+/* $Id: nbase.h 9839 2008-08-29 20:56:53Z david $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -122,7 +122,7 @@
  *   will generally be one of the following: LINUX, FREEBSD, NETBSD,
  *   OPENBSD, SOLARIS, SUNOS, BSDI, IRIX, NETBSD
  *
- * * Insures that GNU getopt_* functions exist (such as getopt_long_only
+ * * Insures that getopt_* functions exist (such as getopt_long_only)
  *
  * * Various string functions such as Strncpy() and strcasestr() see protos 
  *   for more info.
@@ -185,37 +185,18 @@
 
 #include <stdarg.h>
 
-/* Integer widths */
-#if (SIZEOF_CHAR == 1)
-typedef unsigned char u8;
-#else
-typedef u_int8_t u8;
-#endif
+/* Keep assert() defined for security reasons */
+#undef NDEBUG
 
-#if (SIZEOF_SHORT == 2)
-typedef unsigned short u16;
-typedef short s16;
-#elif (SIZEOF_CHAR == 2)
-typedef unsigned char u16;
-typedef char s16;
-#else
-typedef u_int16_t u16;
+/* Integer types */
+typedef uint8_t u8;
+typedef int8_t s8;
+typedef uint16_t u16;
 typedef int16_t s16;
-#endif
-
-#if (SIZEOF_SHORT == 4)
-typedef unsigned short u32;
-typedef short s32;
-#elif (SIZEOF_INT == 4)
-typedef unsigned int u32;
-typedef int s32;
-#elif (SIZEOF_LONG == 4)
-typedef unsigned long u32;
-typedef long s32;
-#else
-typedef u_int32_t u32;
+typedef uint32_t u32;
 typedef int32_t s32;
-#endif
+typedef uint64_t u64;
+typedef int64_t s64;
 
 /* Mathematicial MIN/MAX/ABS (absolute value) macros */
 #ifndef MAX
@@ -270,17 +251,12 @@ extern "C" int snprintf (char *, size_t, const char *, ...);
 extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #endif
 
-/* GNU getopt replacements ... Anyone have a BSD licensed version of these? */
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #else
-/* The next half-dozen lines are from gcc-2.95 ... -Fyodor */
-/* Include getopt.h for the sake of getopt_long.
-   We don't need the declaration of getopt, and it could conflict
-   with something from a system header file, so effectively nullify that.  */
-#define getopt getopt_loser
+#ifndef HAVE_GETOPT_LONG_ONLY
 #include "getopt.h"
-#undef getopt
+#endif
 #endif /* HAVE_GETOPT_H */
 
 /* More Windows-specific stuff */
@@ -293,6 +269,18 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 #define S_ISDIR(m)      (((m) & _S_IFMT) == _S_IFDIR)
 #endif
 
+/* Windows doesn't have the access() defines */
+#ifndef F_OK
+#define F_OK 00
+#endif
+#ifndef W_OK
+#define W_OK 02
+#endif
+#ifndef R_OK
+#define R_OK 04
+#endif
+
+#define access _access
 #define stat _stat /* wtf was ms thinking? */
 #define execve _execve
 #define getpid _getpid
@@ -306,7 +294,6 @@ extern "C" int vsnprintf (char *, size_t, const char *, va_list);
 
 #if !defined(__GNUC__)
 #define snprintf _snprintf
-#define vsnprintf _vsnprintf
 #endif
 
 #define strcasecmp _stricmp
@@ -373,6 +360,10 @@ unsigned int sleep(unsigned int seconds);
    it must truncate */
 int Strncpy(char *dest, const char *src, size_t n);
 
+int Vsnprintf(char *, size_t, const char *, va_list)
+     __attribute__ ((format (printf, 3, 0)));
+int Snprintf(char *, size_t, const char *, ...)
+     __attribute__ ((format (printf, 3, 4)));
 
 /* Trivial function that returns nonzero if all characters in str of
    length strlength are printable (as defined by isprint()) */
@@ -381,6 +372,10 @@ int stringisprintable(const char *str, int strlength);
 /* Convert non-printable characters to replchar in the string */
 void replacenonprintable(char *str, int strlength, char replchar);
 
+/* Portable, incompatible replacements for dirname and basename. */
+char *path_get_dirname(const char *path);
+char *path_get_basename(const char *path);
+
 /* A few simple wrappers for the most common memory allocation routines which will exit() if the
 	allocation fails, so you don't always have to check -- see nbase_memalloc.c */
 void *safe_malloc(size_t size);
@@ -398,9 +393,14 @@ u32 get_random_u32();
 u16 get_random_u16();
 u8 get_random_u8();
 
+int unblock_socket(int sd);
+int block_socket(int sd);
+
 /* CRC16 Cyclic Redundancy Check  */
 unsigned long crc16(unsigned char *buf, int len);
 
+long tval2msecs(char *tspec);
+
 #ifndef STDIN_FILENO
 #define STDIN_FILENO 0
 #endif
