@@ -5,7 +5,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -24,7 +24,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -38,7 +38,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -59,7 +59,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -77,7 +77,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -91,13 +91,15 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_misc.c 3899 2006-08-29 05:42:35Z fyodor $ */
+/* $Id: nbase_misc.c 9700 2008-08-23 18:45:30Z david $ */
+
+#include "nbase.h"
 
 #ifndef WIN32
 #include <errno.h>
@@ -108,11 +110,11 @@ extern int errno;
 #include <winsock2.h>
 #endif
 
-#include <assert.h>
-
-#include "nbase.h"
 #include "nbase_ipv6.h"
 
+#include <assert.h>
+#include <fcntl.h>
+
 #ifndef INET6_ADDRSTRLEN
 #define INET6_ADDRSTRLEN 46
 #endif
@@ -151,17 +153,70 @@ const char *inet_ntop_ez(struct sockaddr_storage *ss, size_t sslen) {
     return inet_ntop(AF_INET, &sin->sin_addr, str, sizeof(str));
   } 
 #if HAVE_IPV6
-  else {
-    assert(sin->sin_family == AF_INET6);
+  else if(sin->sin_family == AF_INET6) {
     if (sslen < sizeof(struct sockaddr_in6))
       return NULL;
     return inet_ntop(AF_INET6, &sin6->sin6_addr, str, sizeof(str));
   } 
 #endif
-
+  //Some laptops report the ip and address family of disabled wifi cards as null
+  //so yes, we will hit this sometimes.
   return NULL;
 }
 
+int unblock_socket(int sd) {
+#ifdef WIN32
+u_long one = 1;
+if(sd != 501) // Hack related to WinIP Raw Socket support
+  ioctlsocket (sd, FIONBIO, &one);
+#else
+int options;
+/*Unblock our socket to prevent recvfrom from blocking forever
+  on certain target ports. */
+options = O_NONBLOCK | fcntl(sd, F_GETFL);
+fcntl(sd, F_SETFL, options);
+#endif //WIN32
+return 1;
+}
+
+/* Convert a socket to blocking mode */
+int block_socket(int sd) {
+#ifdef WIN32
+  unsigned long options=0;
+  if(sd == 501) return 1;
+  ioctlsocket(sd, FIONBIO, &options);
+#else
+  int options;
+  options = (~O_NONBLOCK) & fcntl(sd, F_GETFL);
+  fcntl(sd, F_SETFL, options);
+#endif
+
+  return 1;
+}
+
+/* Converts a time specification string into milliseconds.  If the string
+ * is a plain non-negative number, it is considered to already be in
+ * milliseconds and is returned.  If it is a number followed by 's' (for
+ * seconds), 'm' (minutes), or 'h' (hours), the number is converted to
+ * milliseconds and returned.  If it cannot parse the string, -1 is
+ * returned instead.
+ */
+long tval2msecs(char *tspec) {
+  long l;
+  char *endptr = NULL;
+  l = strtol(tspec, &endptr, 10);
+  if (l < 0 || !endptr) return -1;
+  if (*endptr == '\0') return l;
+  if (*endptr == 's' || *endptr == 'S') return l * 1000;
+  if ((*endptr == 'm' || *endptr == 'M')) {
+    if (*(endptr + 1) == 's' || *(endptr + 1) == 'S') 
+      return l;
+    return l * 60000;
+  }
+  if (*endptr == 'h' || *endptr == 'H') return l * 3600000;
+  return -1;
+}
+
 
 /*
  * CRC16 Cyclic Redundancy Check
