@@ -4,7 +4,7 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2006 Insecure.Com LLC. Nmap is    *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
  * also a registered trademark of Insecure.Com LLC.  This program is free  *
  * software; you may redistribute and/or modify it under the terms of the  *
  * GNU General Public License as published by the Free Software            *
@@ -23,7 +23,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -37,7 +37,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://insecure.org/nmap/ to download Nmap.                             *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -58,7 +58,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -76,7 +76,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -90,17 +90,29 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase_winunix.h 3899 2006-08-29 05:42:35Z fyodor $ */
+/* $Id: nbase_winunix.h 9933 2008-09-04 14:18:21Z david $ */
 
 #ifndef NBASE_WINUNIX_H
 #define NBASE_WINUNIX_H
 
+#include "nbase_winconfig.h"
+
+/* Define the earliest version of Windows we support.  These control
+   what parts of the Windows API are available. The available constants
+   are in <sdkddkver.h>.
+   http://msdn.microsoft.com/en-us/library/aa383745.aspx
+   http://blogs.msdn.com/oldnewthing/archive/2007/04/11/2079137.aspx */
+#undef _WIN32_WINNT
+#define _WIN32_WINNT _WIN32_WINNT_WIN2K
+#undef NTDDI_VERSION
+#define NTDDI_VERSION NTDDI_WIN2KSP4
+
 #define _INC_ERRNO  /* supress errno.h */
 #define _ERRNO_H_ /* Also for errno.h suppresion */
 
@@ -111,6 +123,11 @@
 #include <windows.h>
 #include <winsock2.h>
 #include <ws2tcpip.h> /* IPv6 stuff */
+#if HAVE_WSPIAPI_H
+/* <wspiapi.h> is necessary for getaddrinfo before Windows XP, but it isn't
+   available on some platforms like MinGW. */
+#include <wspiapi.h>
+#endif
 #include <time.h>
 #include <iptypes.h>
 #include <stdlib.h>
@@ -124,24 +141,6 @@
 #include <WINCRYPT.H>
 #include <math.h>
 
-#ifndef HAVE_U_INT8_T
-#define HAVE_U_INT8_T
-typedef unsigned char u_int8_t;
-#endif
-
-#ifndef HAVE_U_INT16_T
-#define HAVE_U_INT16_T
-typedef unsigned short u_int16_t;
-#endif
-
-#ifndef HAVE_U_INT32_T
-#define HAVE_U_INT32_T
-typedef unsigned long u_int32_t;
-#endif
-
-#ifndef _SSIZE_T_
-typedef unsigned int ssize_t;
-#endif
 
 #define SIOCGIFCONF     0x8912          /* get iface list */
 
@@ -164,6 +163,7 @@ int nmapwin_isroot();
 #define ECONNREFUSED    WSAECONNREFUSED
 #undef  EAGAIN
 #define EAGAIN		WSAEWOULDBLOCK
+#define EWOULDBLOCK	WSAEWOULDBLOCK
 #define EHOSTUNREACH	WSAEHOSTUNREACH
 #define ENETDOWN	WSAENETDOWN
 #define ENETUNREACH	WSAENETUNREACH
@@ -185,18 +185,14 @@ int nmapwin_isroot();
 #define EMSGSIZE        WSAEMSGSIZE    /* Message too long */
 #undef  ENOMEM
 #define ENOMEM          WSAENOBUFS
+#undef  ENOTSOCK
+#define ENOTSOCK        WSAENOTSOCK
 #undef  EIO
 #define EIO             WSASYSCALLFAILURE
 
-#define close(x) my_close(x)
+#define close(x) closesocket(x)
 /* #define read(x,y,z) recv(x,(char*)(y),z,0) */
 
-#ifdef __cplusplus
-  extern "C" int my_close(int sd);
-#else
-  int my_close(int sd);
-#endif
-
 typedef unsigned short u_short_t;
 
 #endif /* NBASE_WINUNIX_H */
