@@ -23,7 +23,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -58,7 +58,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -90,9 +90,9 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
@@ -240,6 +240,8 @@ static int read_timeouts[][4] = {
 // Size of hash table used to hold the hosts from /etc/hosts
 #define HASH_TABLE_SIZE 256
 
+// Hash macro for etchosts
+#define IP_HASH(x) (ntohl(x)%HASH_TABLE_SIZE)
 
 
 //------------------- Internal Structures ---------------------
@@ -285,6 +287,7 @@ static std::list<request *> cname_reqs;
 static int total_reqs;
 static nsock_pool dnspool=NULL;
 
+/* The DNS cache, not just for entries from /etc/hosts. */
 static std::list<host_elem *> etchosts[HASH_TABLE_SIZE];
 
 static int stat_actual, stat_ok, stat_nx, stat_sf, stat_trans, stat_dropped, stat_cname;
@@ -854,6 +857,14 @@ static void connect_dns_servers() {
     s = *serverI;
 
     s->nsd = nsi_new(dnspool, NULL);
+    if (o.spoofsource) {
+      struct sockaddr_storage ss;
+      size_t sslen;
+      o.SourceSockAddr(&ss, &sslen);
+      nsi_set_localaddr(s->nsd, &ss, sslen);
+    }
+    if (o.ipoptionslen)
+      nsi_set_ipoptions(s->nsd, o.ipoptions, o.ipoptionslen);
     s->reqs_on_wire = 0;
     s->capacity = CAPACITY_MIN;
     s->write_busy = 0;
@@ -964,11 +975,10 @@ static void parse_resolvdotconf() {
 }
 
 
-static void parse_etchosts(char *fname) {
+static void parse_etchosts(const char *fname) {
   FILE *fp;
   char buf[2048], hname[256], ipaddrstr[16], *tp;
   struct in_addr ia;
-  host_elem *he;
 
   fp = fopen(fname, "r");
   if (fp == NULL) return; // silently is OK
@@ -1020,9 +1030,9 @@ bool remove_and_age(host_elem *host) {
      return true;
 }
 
-/* Add to the dns cache. If there is no space we age and 
- * remove the least frequently used entries until there 
- * is space */
+/* Add to the dns cache. If there are too many entries
+ * we age and remove the least frequently used ones to
+ * make more space. */
 static void addto_etchosts(u32 ip, const char *hname) {
   static u16 total_size = 0;
   std::list<host_elem*>::iterator it;
@@ -1036,8 +1046,10 @@ static void addto_etchosts(u32 ip, const char *hname) {
     for(i = 0; i < HASH_TABLE_SIZE; i++) {
       while((it = find_if(etchosts[i].begin(), etchosts[i].end(), remove_and_age)) != etchosts[i].end()) {
         etchosts[i].erase(it);
-        if((total_size--) < HASH_TABLE_SIZE/2)
-		break;
+        /* We don't want total_size to become out of sync with the actual number
+           of entries. */
+        assert(total_size > 0);
+        total_size--;
       }
     }
   }
@@ -1045,7 +1057,7 @@ static void addto_etchosts(u32 ip, const char *hname) {
   he->name = strdup(hname);
   he->addr = ip;
   he->cache_hits = 0;
-  etchosts[ip % HASH_TABLE_SIZE].push_back(he);
+  etchosts[IP_HASH(ip)].push_back(he);
   total_size++;
 }
 
@@ -1054,8 +1066,8 @@ static void addto_etchosts(u32 ip, const char *hname) {
 static char *lookup_etchosts(u32 ip) {
   std::list<host_elem *>::iterator hostI;
   host_elem *tpelem;
-
-  for(hostI = etchosts[ip % HASH_TABLE_SIZE].begin(); hostI != etchosts[ip % HASH_TABLE_SIZE].end(); hostI++) {
+  int localIP_Hash = IP_HASH(ip);
+  for(hostI = etchosts[localIP_Hash].begin(); hostI != etchosts[localIP_Hash].end(); hostI++) {
     tpelem = *hostI;
     if (tpelem->addr == ip) {
       if(tpelem->cache_hits < UCHAR_MAX)
@@ -1069,7 +1081,7 @@ static char *lookup_etchosts(u32 ip) {
 /* External interface to dns cache */
 const char *lookup_cached_host(u32 ip) {
   const char *tmp = lookup_etchosts(ip);
-  return tmp==NULL?"":tmp;
+  return tmp;
 }
 
 static void etchosts_init(void) {
@@ -1160,7 +1172,7 @@ static void nmap_mass_rdns_core(Target **targets, int num_targets) {
     if (o.dns_servers) add_dns_server(o.dns_servers);
     else parse_resolvdotconf();
 
-    if (servs.size() == 0 && firstrun) error("mass_dns: warning: Unable to determine any DNS servers. Reverse DNS is disabled. Try using --system-dns or specify valid servers with --dns_servers");
+    if (servs.size() == 0 && firstrun) error("mass_dns: warning: Unable to determine any DNS servers. Reverse DNS is disabled. Try using --system-dns or specify valid servers with --dns-servers");
   }
 
 
@@ -1314,3 +1326,21 @@ void nmap_mass_rdns(Target **targets, int num_targets) {
 
   firstrun=0;
 }
+
+
+// Returns a list of known DNS servers
+std::list<std::string> get_dns_servers() {
+  // if, for example, run with -n, list is not initialized,
+  // run empty nmap_mass_rdns to do so
+  if(servs.size() == 0 && firstrun) {
+    nmap_mass_rdns(NULL, 0);
+  }
+  std::list<dns_server *>::iterator servI;
+  std::list<std::string> serverList;
+  dns_server *tpserv;
+  for(servI = servs.begin(); servI != servs.end(); servI++) {
+    tpserv = *servI;
+    serverList.push_back(inet_ntoa(tpserv->addr.sin_addr));
+  }
+  return serverList;
+}
