@@ -1,23 +1,23 @@
 
 /***************************************************************************
- * rpc.cc -- Functions related to the RPCGrind (-sR) facility of Nmap.     *
+ * nmap_rpc.cc -- Functions related to the RPCGrind (-sR) facility of Nmap *
  * This includes reading the nmap-rpc services file and sending rpc        *
  * queries and interpreting responses.  The actual scan engine used for    *
  * rpc grinding is pos_scan (which is not in this file)                    *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -26,7 +26,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -40,7 +40,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -52,16 +52,16 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -79,7 +79,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -93,17 +93,22 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_rpc.cc 3200 2006-03-05 23:59:46Z fyodor $ */
+/* $Id: nmap_rpc.cc 7640 2008-05-22 20:45:32Z fyodor $ */
 
 
 #include "nmap_rpc.h"
 #include "NmapOps.h"
+#include "Target.h"
+#include "charpool.h"
+#include "timing.h"
+#include "nmap_error.h"
+#include "utils.h"
 
 extern NmapOps o;
 static struct rpc_info ri;
@@ -133,7 +138,7 @@ static void rpc_services_init() {
   ri.names = (char **) cp_alloc(ri.num_alloc * sizeof(char *));
   ri.numbers = (unsigned long *) cp_alloc(ri.num_alloc * sizeof(unsigned long));
 
-  if (nmap_fetchfile(filename, sizeof(filename), "nmap-rpc") == -1) {
+  if (nmap_fetchfile(filename, sizeof(filename), "nmap-rpc") != 1) {
     error("Unable to find nmap-rpc!  Resorting to /etc/rpc");
     strcpy(filename, "/etc/rpc");
   }
@@ -142,6 +147,8 @@ static void rpc_services_init() {
   if (!fp) {
     fatal("Unable to open %s for reading rpc information", filename);
   }
+  /* Record where this data file was found. */
+  o.loaded_data_files["nmap-rpc"] = filename;
 
   while(fgets(line, sizeof(line), fp)) {
     lineno++;
@@ -205,13 +212,17 @@ int get_rpc_procs(unsigned long **programs, unsigned long *num_programs) {
 /* Send an RPC query to the specified host/port on the specified protocol
    looking for the specified RPC program.  We cache our sending sockets
    to avoid recreating and (with TCP) reconnect()'ing them each time */
-int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
+int send_rpc_query(Target *target_host, unsigned short portno,
 		   int ipproto, unsigned long program, int scan_offset, 
 		   int trynum) {
-  static struct in_addr last_target_host;
+  static struct sockaddr_storage last_target;
+  struct sockaddr_storage sock;
+  size_t socklen;
   static int last_ipproto = -1;
-  static unsigned short last_portno = 0; 
-  struct sockaddr_in sock;
+  struct sockaddr_in *sin = NULL, *lastsin = NULL;
+#ifdef HAVE_IPV6
+  struct sockaddr_in6 *sin6 = NULL, *lastsin6 = NULL;
+#endif
   char rpch_buf[256]; 
   struct rpc_hdr *rpch;
   int res, err = 0;
@@ -228,41 +239,60 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
     rpc_xid_base = (unsigned long) get_random_uint();
   
   if (o.debugging > 1) {
-    printf("Sending RPC probe for program %li to %hu/%s -- scan_offset=%d trynum=%d xid=%lX\n", program, portno, proto2ascii(ipproto), scan_offset, trynum, rpc_xid_base + ((portno & 0x3FFF) << 16) + (trynum << 30) +  scan_offset);
+    log_write(LOG_PLAIN, "Sending RPC probe for program %li to %hu/%s -- scan_offset=%d trynum=%d xid=%lX\n", program, portno, proto2ascii(ipproto), scan_offset, trynum, rpc_xid_base + ((portno & 0x3FFF) << 16) + (trynum << 30) +  scan_offset);
   }
 
+  memset(&sock, 0, sizeof(sock));
+  target_host->TargetSockAddr(&sock, &socklen);
+
+  if (sock.ss_family == AF_INET) {
+    sin = (struct sockaddr_in *) &sock;
+    lastsin = (struct sockaddr_in *) &last_target;
+
+    sin->sin_port = htons(portno);
+  }
+#ifdef HAVE_IPV6
+  else {
+    sin6 = (struct sockaddr_in6 *) &sock;
+    lastsin6 = (struct sockaddr_in6 *) &last_target;
+
+    sin6->sin6_port = htons(portno);
+  }
+#endif
+
   /* First we check whether we have to create a new connection -- we 
      need to if we have a new target_host, or a new portno, or the socket
      we want to use is -1 */
-  if (ipproto == IPPROTO_TCP && 
-      (last_target_host.s_addr != target_host->s_addr ||
-       last_portno != portno || last_ipproto != IPPROTO_TCP)) {
-    /* New host or port -- kill our old tcp socket */
-    if (tcp_rpc_socket != -1) {
-      close(tcp_rpc_socket);
-      tcp_rpc_socket = -1;
-      tcp_readlen = 0;
+  if (ipproto == IPPROTO_TCP) {
+    if ((sock.ss_family == AF_INET &&
+         memcmp(sin, lastsin, sizeof(struct sockaddr_in)))
+#ifdef HAVE_IPV6
+     || (sock.ss_family == AF_INET6 &&
+         memcmp(sin6, lastsin6, sizeof(struct sockaddr_in6)))
+#endif
+     || last_ipproto != IPPROTO_TCP) {
+      /* New host or port -- kill our old tcp socket */
+      if (tcp_rpc_socket != -1) {
+        close(tcp_rpc_socket);
+        tcp_rpc_socket = -1;
+        tcp_readlen = 0;
+      }
     }
   }
-  last_ipproto = ipproto;
-  last_target_host.s_addr = target_host->s_addr;
-  last_portno = portno;
   
-  memset(&sock, 0, sizeof(sock));
-  sock.sin_family = AF_INET;
-  sock.sin_addr.s_addr = target_host->s_addr;
-  sock.sin_port = htons(portno);
-    
+  last_target = sock;
+  last_ipproto = ipproto;
+
   if (ipproto == IPPROTO_TCP && tcp_rpc_socket == -1) {
-    if ((tcp_rpc_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
-      pfatal("Socket troubles in send_rpc_query");
+    if ((tcp_rpc_socket = socket(sock.ss_family, SOCK_STREAM, IPPROTO_TCP)) == -1)
+      pfatal("Socket troubles in %s", __func__);
     /* I should unblock the socket here and timeout the connect() */
     res = connect(tcp_rpc_socket, (struct sockaddr *) &sock, 
-		  sizeof(struct sockaddr_in));
+		  sizeof(struct sockaddr_storage));
     if (res == -1) {
       if (o.debugging) {
-	gh_perror("Failed to connect to port %d of %s in send_rpc_query",
-		  portno, inet_ntoa(*target_host));
+	gh_perror("Failed to connect to port %d of %s in %s",
+		  portno, target_host->targetipstr(), __func__);
       }
       close(tcp_rpc_socket);
       tcp_rpc_socket = -1;
@@ -270,8 +300,8 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
     }
     unblock_socket(tcp_rpc_socket);
   } else if (ipproto == IPPROTO_UDP && udp_rpc_socket == -1) {
-    if ((udp_rpc_socket = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
-      pfatal("UDP socket troubles in send_rpc_query");
+    if ((udp_rpc_socket = socket(sock.ss_family, SOCK_DGRAM, 0)) == -1)
+      pfatal("UDP socket troubles in %s", __func__);
     unblock_socket(udp_rpc_socket);
   }
   
@@ -299,7 +329,7 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
       if (o.debugging > 1)
 	hdump((unsigned char *) rpch, sizeof(struct rpc_hdr));
       res = sendto(udp_rpc_socket, (char *)rpch, sizeof(struct rpc_hdr), 0,
-		   (struct sockaddr *) &sock, sizeof(struct sockaddr_in));
+		   (struct sockaddr *) &sock, sizeof(struct sockaddr_storage));
       if (res == -1)
 	err = socket_errno();
      } while(res == -1 && (err == EINTR || err == ENOBUFS));
@@ -307,7 +337,7 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
 
     if (res == -1) {
       if (o.debugging) {
-	gh_perror("Sendto in send_rpc_query");
+	gh_perror("Sendto in %s", __func__);
 	close(udp_rpc_socket);
 	udp_rpc_socket = -1;
       }
@@ -320,7 +350,7 @@ int send_rpc_query(const struct in_addr *target_host, unsigned short portno,
     res = Send(tcp_rpc_socket, rpch_buf, sizeof(struct rpc_hdr) + sizeof(unsigned long), 0);
     if (res == -1) {
       if (o.debugging) {
-	gh_perror("Write in send_rpc_query");
+	gh_perror("Write in %s", __func__);
       }
       close(tcp_rpc_socket);
       tcp_rpc_socket = -1;
@@ -350,9 +380,9 @@ static int rpc_are_we_done(char *msg, int msg_len, Target *target,
     /* This is not a valid reply -- we kill the port 
        (from an RPC perspective) */ 
     if (o.debugging > 1) {
-      printf("Port %hu/%s labelled NON_RPC because of invalid sized message (%d)\n", 
-	     rsi->rpc_current_port->portno, 
-	     proto2ascii(rsi->rpc_current_port->proto, true), msg_len);
+      log_write(LOG_PLAIN, "Port %hu/%s labelled NON_RPC because of invalid sized message (%d)\n", 
+		rsi->rpc_current_port->portno, 
+		proto2ascii(rsi->rpc_current_port->proto, true), msg_len);
     }
     rsi->rpc_status = RPC_STATUS_NOT_RPC;
     ss->numqueries_outstanding = 0;
@@ -365,7 +395,7 @@ static int rpc_are_we_done(char *msg, int msg_len, Target *target,
   if (((scan_offset >> 16) & 0x3FFF) != (unsigned long) (rsi->rpc_current_port->portno & 0x3FFF)) {
     /* Doh -- this doesn't seem right */
     if (o.debugging > 1) {
-      printf("Port %hu/%s labelled NON_RPC because ((scan_offset >> 16) & 0x3FFF) is %li\n", rsi->rpc_current_port->portno, proto2ascii(rsi->rpc_current_port->proto, true), ((scan_offset >> 16) & 0x3FFF));
+      log_write(LOG_PLAIN, "Port %hu/%s labelled NON_RPC because ((scan_offset >> 16) & 0x3FFF) is %li\n", rsi->rpc_current_port->portno, proto2ascii(rsi->rpc_current_port->proto, true), ((scan_offset >> 16) & 0x3FFF));
     }
     rsi->rpc_status = RPC_STATUS_NOT_RPC;
     ss->numqueries_outstanding = 0;
@@ -404,7 +434,7 @@ static int rpc_are_we_done(char *msg, int msg_len, Target *target,
   }
      
   if (trynum > current->trynum) {
-    error("Bogus trynum %d when we are only up to %d in get_rpc_results", trynum, current->trynum);
+    error("Bogus trynum %d when we are only up to %d in %s", trynum, current->trynum, __func__);
     rsi->rpc_status = RPC_STATUS_NOT_RPC;
     ss->numqueries_outstanding = 0;
     return 1;
@@ -504,7 +534,7 @@ unsigned long current_msg_len;
    if (tcp_rpc_socket > max_sd)
      max_sd = tcp_rpc_socket;
  } else {
-   error("Unable to find listening socket in get_rpc_results");
+   error("Unable to find listening socket in %s", __func__);
    return;
  }
 
@@ -529,18 +559,18 @@ unsigned long current_msg_len;
      if (res < 0) {
        /* Doh! */
        if (o.debugging || o.verbose)
-	 gh_perror("recvfrom in get_rpc_results");
+	 gh_perror("recvfrom in %s", __func__);
        ss->numqueries_outstanding = 0;
        rsi->rpc_status = RPC_STATUS_NOT_RPC;
        return;
      }
      if (o.debugging > 1)
-       printf("Received %d byte UDP packet\n", res);
+       log_write(LOG_PLAIN, "Received %d byte UDP packet\n", res);
      /* Now we check that the response is from the expected host/port */
      if (from.sin_addr.s_addr != target->v4host().s_addr ||
 	 from.sin_port != htons(rsi->rpc_current_port->portno)) {
        if (o.debugging > 1) {
-	 printf("Received UDP packet from %d.%d.%d.%d/%hu when expecting packet from %d.%d.%d.%d/%hu\n", NIPQUAD(from.sin_addr.s_addr), ntohs(from.sin_port), NIPQUAD(target->v4host().s_addr), rsi->rpc_current_port->portno);
+	 log_write(LOG_PLAIN, "Received UDP packet from %d.%d.%d.%d/%hu when expecting packet from %d.%d.%d.%d/%hu\n", NIPQUAD(from.sin_addr.s_addr), ntohs(from.sin_port), NIPQUAD(target->v4host().s_addr), rsi->rpc_current_port->portno);
        }
        continue;
      }
@@ -555,9 +585,9 @@ unsigned long current_msg_len;
      if (res <= 0) {
        if (o.debugging) {
 	 if (res == -1)
-	   gh_perror("Failed to read() from tcp rpc socket in get_rpc_results");
+	   gh_perror("Failed to read() from tcp rpc socket in %s", __func__);
 	 else {
-	   error("Lamer on port %u closed RPC socket on me in get_rpc_results", rsi->rpc_current_port->portno);
+	   error("Lamer on port %u closed RPC socket on me in %s", rsi->rpc_current_port->portno, __func__);
 	 }
        }
        ss->numqueries_outstanding = 0;
@@ -571,10 +601,10 @@ unsigned long current_msg_len;
        /* This is suspiciously small -- I'm assuming this is not the first
 	  part of a valid RPC packet */
        if (o.debugging > 1) {
-	 printf("Port %hu/%s labelled NON_RPC because tcp_readlen is %d (should be at least 28)\n", 
-		rsi->rpc_current_port->portno, 
-		proto2ascii(rsi->rpc_current_port->proto, true), 
-		(int) tcp_readlen);
+	 log_write(LOG_PLAIN, "Port %hu/%s labelled NON_RPC because tcp_readlen is %d (should be at least 28)\n", 
+		   rsi->rpc_current_port->portno, 
+		   proto2ascii(rsi->rpc_current_port->proto, true), 
+		   (int) tcp_readlen);
        }
        ss->numqueries_outstanding = 0;
        rsi->rpc_status = RPC_STATUS_NOT_RPC;
@@ -585,9 +615,10 @@ unsigned long current_msg_len;
 						     
      if (current_msg_len > tcp_readlen - 4) {
        if (o.debugging > 1) {
-	 printf("Port %hu/%s labelled NON_RPC because current_msg_len is %li while tcp_readlen is %d\n", rsi->rpc_current_port->portno, 
-		proto2ascii(rsi->rpc_current_port->proto, true), 
-		current_msg_len, (int) tcp_readlen);
+	 log_write(LOG_PLAIN, "Port %hu/%s labelled NON_RPC because current_msg_len is %li while tcp_readlen is %d\n",
+		   rsi->rpc_current_port->portno, 
+		   proto2ascii(rsi->rpc_current_port->proto, true), 
+		   current_msg_len, (int) tcp_readlen);
        }
        ss->numqueries_outstanding = 0;
        rsi->rpc_status = RPC_STATUS_NOT_RPC;
@@ -615,10 +646,10 @@ unsigned long current_msg_len;
        if (current_msg_len < 24 || current_msg_len > 32) {
 	 ss->numqueries_outstanding = 0;
 	 if (o.debugging > 1) {
-	   printf("Port %hu/%s labelled NON_RPC because current_msg_len is %li\n", 
-		  rsi->rpc_current_port->portno, 
-		  proto2ascii(rsi->rpc_current_port->proto, true), 
-		  current_msg_len);
+	   log_write(LOG_PLAIN, "Port %hu/%s labelled NON_RPC because current_msg_len is %li\n", 
+		     rsi->rpc_current_port->portno, 
+		     proto2ascii(rsi->rpc_current_port->proto, true), 
+		     current_msg_len);
 	 }
 	 rsi->rpc_status = RPC_STATUS_NOT_RPC;
 	 return;
@@ -650,12 +681,3 @@ void close_rpc_query_sockets() {
   }
 }
 
-
-
-
-
-
-
-
-
-
