@@ -7,13 +7,13 @@ extern "C" {
 #include "lauxlib.h"
 }
 
-int l_nsock_open(lua_State* l);
-int l_nsock_new(lua_State* l);
+int luaopen_nsock(lua_State *);
+int l_nsock_new(lua_State *);
 int l_nsock_loop(int tout);
 
-int l_dnet_new(lua_State* l);
-int l_dnet_open(lua_State* l);
-int l_dnet_get_interface_link(lua_State* l);
+int l_dnet_new(lua_State *);
+int l_dnet_open(lua_State *);
+int l_dnet_get_interface_link(lua_State *);
 
 #endif
 
