@@ -17,7 +17,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -35,7 +35,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * insecure.org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -49,12 +49,12 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: filespace.h 6635 2007-12-22 06:32:18Z fyodor $ */
+/* $Id: filespace.h 8430 2008-06-22 04:56:02Z bmenrigh $ */
 
 #ifndef FILESPACE_H
 #define FILESPACE_H
@@ -95,7 +95,7 @@ static inline int fs_rputs(const char *str, struct filespace *fs) {
 
   if (len + fs->current_size + 1 > fs->current_alloc) {
     fs->current_alloc =  MAX(fs->current_size * 2, fs->current_size + len  + 1000);
-    new_str = (char *) malloc(fs->current_alloc);
+    new_str = (char *) safe_malloc(fs->current_alloc);
     memcpy(new_str, fs->str, fs->current_size);
     fs->pos = (fs->pos - fs->str) + new_str;
     if (fs->str)  
@@ -141,9 +141,7 @@ if (len == 0) return 0;
   if (fs->current_alloc - fs->current_size < len + 2) {
     fs->current_alloc = (int) (fs->current_alloc * 1.4 + 1 );
     fs->current_alloc += 100 + len;
-    tmpstr = (char *) malloc(fs->current_alloc);
-    if (!tmpstr)
-      return -1;
+    tmpstr = (char *) safe_malloc(fs->current_alloc);
     memcpy(tmpstr, fs->str, fs->current_size);
     fs->pos = (fs->pos - fs->str) + tmpstr;
     if (fs->str) free(fs->str);
