@@ -18,7 +18,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -50,12 +50,12 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_internal.h 6859 2008-02-28 18:52:17Z fyodor $ */
+/* $Id: nsock_internal.h 8885 2008-07-14 19:37:26Z kris $ */
 
 #ifndef NSOCK_INTERNAL_H
 #define NSOCK_INTERNAL_H
@@ -206,10 +206,11 @@ struct msiod {
   enum msiod_state state;
   struct sockaddr_storage peer; /* The host and port we are connected to
 				   using sd (saves a call to getpeername) */
+  struct sockaddr_storage local; /* The host and port to bind to with sd */
 
-/* The length of peer actualy used (sizeof(sockadd_in) or
-   sizeof(sockaddr_in6), or 0 if peer has not been filled in */
-  size_t peerlen; 
+/* The length of peer/local actually used (sizeof(sockaddr_in) or
+   sizeof(sockaddr_in6), or 0 if peer/local has not been filled in */
+  size_t locallen, peerlen; 
   int lastproto; /* -1 if none yet, otherwise IPPROTO_TCP, etc. */
   gh_list_elem *entry_in_nsp_active_iods; /* The mspool keeps track of
 					     msiods that have been
@@ -229,6 +230,10 @@ struct msiod {
   unsigned long id; /* Every iod has an id which is always unique for the
 		       same nspool (unless you create billions of them) */
   void *userdata;
+
+  /* IP options to set on socket before connect() */
+  void *ipopts;
+  int ipoptslen;
   
   void *pcap;	   /* Pointer to mspcap struct (used only if pcap support is included) */
 };
