@@ -7,7 +7,7 @@
  *                                                                         *
  ***********************IMPORTANT NSOCK LICENSE TERMS***********************
  *                                                                         *
- * The nsock parallel socket event library is (C) 1999-2004 Insecure.Com   *
+ * The nsock parallel socket event library is (C) 1999-2008 Insecure.Com   *
  * LLC This library is free software; you may redistribute and/or          *
  * modify it under the terms of the GNU General Public License as          *
  * published by the Free Software Foundation; Version 2.  This guarantees  *
@@ -19,7 +19,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -37,7 +37,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * insecure.org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -51,18 +51,22 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details (                               *
- * http://www.gnu.org/copyleft/gpl.html ).                                 *
+ * General Public License v2.0 for more details                            *
+ * (http://www.gnu.org/licenses/gpl-2.0.html).                             *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nsock_iod.c 3439 2006-06-10 21:23:36Z fyodor $ */
+/* $Id: nsock_iod.c 9270 2008-08-02 03:44:44Z kris $ */
 
 #include "nsock.h"
 #include "nsock_internal.h"
 #include "gh_list.h"
 #include "netutils.h"
 
+#if HAVE_PCAP
+#include "nsock_pcap.h"
+#endif
+
 #include <string.h>
 
 /* nsock_iod is like a "file descriptor" for the nsock library.  You
@@ -99,14 +103,19 @@ nsock_iod nsi_new2(nsock_pool nsockp, int sd, void *userdata) {
       free(nsi);
       return NULL;
     }
-    nsock_unblock_socket(nsi->sd);
+    unblock_socket(nsi->sd);
     nsi->state = NSIOD_STATE_UNKNOWN;
   }
+
+  nsi->locallen = 0;
     
   nsi->userdata = userdata;
   nsi->nsp = (mspool *) nsockp;
   nsi->events_pending = 0;
 
+  nsi->ipopts = NULL;
+  nsi->ipoptslen = 0;
+
 #if HAVE_OPENSSL
   nsi->ssl_session = NULL;
 #endif
@@ -162,8 +171,8 @@ void nsi_delete(nsock_iod nsockiod, int pending_response) {
 	currev = (msevent *) GH_LIST_ELEM_DATA(currev_elem);
 	next_elem = GH_LIST_ELEM_NEXT(currev_elem);
 	if (currev->iod == nsi) {
-	  // OK - we found an event pending on this IOD.  Kill it.
-	  // printf("Found an outstanding event (out of %d), removing\n", nsi->events_pending);
+	  /* OK - we found an event pending on this IOD.  Kill it. */
+	  /* printf("Found an outstanding event (out of %d), removing\n", nsi->events_pending); */
 	  msevent_cancel(nsi->nsp, currev, elist_ar[elist], currev_elem, pending_response == NSOCK_PENDING_NOTIFY);
 	}
 	if (nsi->events_pending == 0)
@@ -207,6 +216,29 @@ void nsi_delete(nsock_iod nsockiod, int pending_response) {
   nsi->state = NSIOD_STATE_DELETED;
   nsi->userdata = NULL;
 
+  if (nsi->ipoptslen)
+    free(nsi->ipopts);
+
+#if HAVE_PCAP
+  if(nsi->pcap){
+    mspcap *mp = (mspcap *) nsi->pcap;
+    if(mp->pt){
+      pcap_close(mp->pt);
+      mp->pt=NULL;
+    }
+    if(mp->pcap_desc){
+      // Should I close pcap_desc or pcap_close does this for me?
+      mp->pcap_desc = -1;
+    }
+    if(mp->pcap_device){
+    	free(mp->pcap_device);
+    	mp->pcap_device = NULL;
+    }
+    free(mp);
+    nsi->pcap = NULL;
+  }
+#endif
+
   gh_list_remove_elem(&nsi->nsp->active_iods, nsi->entry_in_nsp_active_iods);
   gh_list_prepend(&nsi->nsp->free_iods, nsi);
 
@@ -289,6 +321,40 @@ int nsi_peerport(msiod *nsi) {
   return -1;
 }
 
+/* Sets the local address to bind to before connect() */
+int nsi_set_localaddr(nsock_iod nsi, struct sockaddr_storage *ss, size_t sslen)
+{
+	msiod *iod = (msiod *) nsi;
+
+	assert(iod);
+
+	if (sslen > sizeof(iod->local))
+		return -1;
+
+	memcpy(&iod->local, ss, sslen);
+	iod->locallen = sslen;
+	return 0;
+}
+
+/* Sets IPv4 options to apply before connect().  It makes a copy of the 
+ * options, so you can free() yours if necessary.  This copy is freed
+ * when the iod is destroyed
+ */
+int nsi_set_ipoptions(nsock_iod nsi, void *opts, size_t optslen)
+{
+	msiod *iod = (msiod *) nsi;
+
+	assert(iod);
+
+	if (optslen > 44)
+		return -1;
+
+	iod->ipopts = safe_malloc(optslen);
+	memcpy(iod->ipopts, opts, optslen);
+	iod->ipoptslen = optslen;
+	return 0;
+}
+
 /* I didn't want to do this.  Its an ugly hack, but I suspect it will
    be neccessary.  I certainly can't reproduce in nsock EVERYTHING you
    might want to do with a socket.  So I'm offering you this function
