@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -60,7 +60,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -92,13 +92,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: osscan.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: osscan.cc 8908 2008-07-15 05:33:33Z david $ */
 
 #include "osscan.h"
 #include "timing.h"
@@ -629,7 +629,7 @@ static bool FingerTest_lessthan(const FingerTest* a, const FingerTest* b) {
    representation. Tests that are identical between more than one fingerprint
    are included only once. If wrapit is true, the string is wrapped for
    submission. */
-char *mergeFPs(FingerPrint *FPs[], int numFPs, bool isGoodFP,
+const char *mergeFPs(FingerPrint *FPs[], int numFPs, bool isGoodFP,
 			   const struct in_addr * const addr, int distance, const u8 *mac,
 			   int openTcpPort, int closedTcpPort, int closedUdpPort, bool wrapit) {
   static char str[10240];
@@ -760,22 +760,15 @@ return str;
   }
 }
 
-char *fp2ascii(FingerPrint *FP) {
+const char *fp2ascii(FingerPrint *FP) {
 static char str[2048];
 FingerPrint *current;
 struct AVal *AV;
 char *p = str;
-int len;
 memset(str, 0, sizeof(str));
 
 if (!FP) return "(None)";
 
-if(FP->OS_name && *(FP->OS_name)) {
-  len = Snprintf(str, 128, "FingerPrint  %s\n", FP->OS_name);
-  if (len < 0) fatal("OS name too long");
-  p += len;
-}
-
 for(current = FP; current ; current = current->next) {
   Strncpy(p, current->name, sizeof(str) - (p-str));
   p += strlen(p);
@@ -800,108 +793,77 @@ for(current = FP; current ; current = current->next) {
 return str;
 }
 
+/* Return a zero-terminated copy of the substring that starts at p and ends at
+   q, with leading and trailing whitespace stripped. The returned string is
+   allocated with cp_alloc. */
+static char *substrstrip(const char *p, const char *q) {
+  char *s;
+
+  assert(p <= q);
+
+  while (isspace(*p))
+    p++;
+  while (q > p && isspace(*(q - 1)))
+    q--;
+
+  s = (char *) cp_alloc(q - p + 1);
+  memcpy(s, p, q - p);
+  s[q - p] = '\0';
+
+  return s;
+}
+
 /* Parse a 'Class' line found in the fingerprint file into the current
    FP.  Classno is the number of 'class' lines found so far in the
    current fingerprint.  The function quits if there is a parse error */
 static void parse_classline(FingerPrint *FP, char *thisline, int lineno, 
 			    int *classno) {
-  char *p, *q;
+  const char *begin, *end;
+  struct OS_Classification *os_class;
 
-// Wtf????
-  fflush(stdout);
-
-  if (!thisline || strncmp(thisline, "Class ", 6) == 1) {
+  if (!thisline || strncmp(thisline, "Class ", 6) != 0)
     fatal("Bogus line #%d (%s) passed to %s()", lineno, thisline, __func__);
-  }
 
   if (*classno >= MAX_OS_CLASSIFICATIONS_PER_FP)
     fatal("Too many Class lines in fingerprint (line %d: %s), remove some or increase MAX_OS_CLASSIFICATIONS_PER_FP", lineno, thisline);
-  
-  p = thisline + 6;
-  
-  /* First lets get the vendor name */
-  while(*p && isspace(*p)) p++;
-  
-  q = strchr(p, '|');
-  if (!q) {
-    fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
-  }
-  
-  // Trim any trailing whitespace
-  q--;
-  while(isspace(*q)) q--;
-  q++;
-  if (q < p) { fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline); }
-  FP->OS_class[*classno].OS_Vendor = (char *) cp_alloc(q - p + 1);
-  memcpy(FP->OS_class[*classno].OS_Vendor, p, q - p);
-  FP->OS_class[*classno].OS_Vendor[q - p] = '\0';
-  
-  /* Next comes the OS Family */
-  p = q;
-  while(*p && !isalnum(*p)) p++;
 
-  q = strchr(p, '|');
-  if (!q) {
-    fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
-  }
-  // Trim any trailing whitespace
-  q--;
-  while(isspace(*q)) q--;
-  q++;
-  if (q < p) { fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline); }
-  FP->OS_class[*classno].OS_Family = (char *) cp_alloc(q - p + 1);
-  memcpy(FP->OS_class[*classno].OS_Family, p, q - p);
-  FP->OS_class[*classno].OS_Family[q - p] = '\0';
-  
-  /* And now the the OS generation, if available */
-  p = q;
-  while(*p && *p != '|') p++;
-  if (*p) p++;
-  while(*p && isspace(*p) && *p != '|') p++;
-  if (*p == '|') {
-    FP->OS_class[*classno].OS_Generation = NULL;
-    q = p;
-  }
-  else {
-    q = strpbrk(p, " |");
-    if (!q) {
-      fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
-    }
-    // Trim any trailing whitespace
-    q--;
-    while(isspace(*q)) q--;
-    q++;
-    if (q < p) { fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline); }
-    FP->OS_class[*classno].OS_Generation = (char *) cp_alloc(q - p + 1);
-    memcpy(FP->OS_class[*classno].OS_Generation, p, q - p);
-    FP->OS_class[*classno].OS_Generation[q - p] = '\0';
-  }
-  
-  /* And finally the device type */
-  p = q;
-  while(*p && !isalnum(*p)) p++;
-  
-  q = strchr(p, '|');
-  if (!q) {
-    q = p;
-    while(*q) q++;
-  }
-  
-  // Trim any trailing whitespace
-  q--;
-  while(isspace(*q)) q--;
-  q++;
-  if (q < p) { fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline); }
-  FP->OS_class[*classno].Device_Type = (char *) cp_alloc(q - p + 1);
-  memcpy(FP->OS_class[*classno].Device_Type, p, q - p);
-  FP->OS_class[*classno].Device_Type[q - p] = '\0';
-  
+  os_class = &FP->OS_class[*classno];
 
-  //  printf("Got classification #%d for the OS %s: VFGT: %s * %s * %s * %s\n", *classno, FP->OS_name, FP->OS_class[*classno].OS_Vendor, FP->OS_class[*classno].OS_Family, FP->OS_class[*classno].OS_Generation? FP->OS_class[*classno].OS_Generation : "(null)", FP->OS_class[*classno].Device_Type);
+  /* First let's get the vendor name. */
+  begin = thisline + 6;
+  end = strchr(begin, '|');
+  if (end == NULL)
+    fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
+  os_class->OS_Vendor = substrstrip(begin, end);
+
+  /* Next comes the OS family. */
+  begin = end + 1;
+  end = strchr(begin, '|');
+  if (end == NULL)
+    fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
+  os_class->OS_Family = substrstrip(begin, end);
+
+  /* And now the the OS generation. */
+  begin = end + 1;
+  end = strchr(begin, '|');
+  if (end == NULL)
+    fatal("Parse error on line %d of fingerprint: %s\n", lineno, thisline);
+  /* OS generation is handled specially: instead of an empty string it's
+     supposed to be NULL. */
+  while (isspace(*begin))
+    begin++;
+  if (begin < end)
+    os_class->OS_Generation = substrstrip(begin, end);
+  else
+    os_class->OS_Generation = NULL;
+
+  /* And finally the device type. We look for '\0' instead of '|'. */
+  begin = end + 1;
+  end = strchr(begin, '\0');
+  os_class->Device_Type = substrstrip(begin, end);
 
   (*classno)++;
   FP->num_OS_Classifications++;
-
 }
 
 /* Parses a single fingerprint from the memory region given.  If a
@@ -936,20 +898,20 @@ FingerPrint *parse_single_fingerprint(char *fprint_orig) {
     }
 
     if (strncmp(thisline, "Fingerprint ", 12) == 0) {
-      p = thisline + 12;
-      while(*p && isspace((int) *p)) p++;
+      /* Ignore a second Fingerprint line if it appears. */
+      if (FP->OS_name == NULL) {
+        p = thisline + 12;
+        while(*p && isspace((int) *p)) p++;
 
-      q = strchr(p, '\n');
-      if (!q) q = p + strlen(p);
-      while(isspace(*(--q)))
-	;
+        q = strchr(p, '\n');
+        if (!q) q = p + strlen(p);
+        while(q > p && isspace(*(--q)))
+          ;
 
-      if (q < p) fatal("Parse error on line %d of fingerprint: %s", lineno, nextline);
-
-      FP->OS_name = (char *) cp_alloc(q - p + 2);
-      memcpy(FP->OS_name, p, q - p + 1);
-      FP->OS_name[q - p + 1] = '\0';
-      
+        FP->OS_name = (char *) cp_alloc(q - p + 2);
+        memcpy(FP->OS_name, p, q - p + 1);
+        FP->OS_name[q - p + 1] = '\0';
+      }
     } else if (strncmp(thisline, "Class ", 6) == 0) {
 
       parse_classline(FP, thisline, lineno, &classno);
@@ -1025,12 +987,12 @@ void free_fingerprint_file(FingerPrintDB *DB) {
 }
 
 
-FingerPrintDB *parse_fingerprint_file(char *fname) {
+FingerPrintDB *parse_fingerprint_file(const char *fname) {
 FingerPrintDB *DB = NULL;
 FingerPrint *current;
 FILE *fp;
 int max_records = 4096; 
-char line[512];
+char line[2048];
 int numrecords = 0;
 int lineno = 0;
  bool parsingMatchPoints = false;
@@ -1139,7 +1101,7 @@ while(fgets(line, sizeof(line), fp)) {
  return DB;
 }
 
-FingerPrintDB *parse_fingerprint_reference_file(char *dbname) {
+FingerPrintDB *parse_fingerprint_reference_file(const char *dbname) {
 char filename[256];
 
 if (nmap_fetchfile(filename, sizeof(filename), dbname) != 1){
