@@ -1,21 +1,20 @@
-
 /***************************************************************************
  * portlist.cc -- Functions for manipulating various lists of ports        *
  * maintained internally by Nmap.                                          *
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
- * is also a registered trademark of Insecure.Com LLC.  This program is    *
- * free software; you may redistribute and/or modify it under the          *
- * terms of the GNU General Public License as published by the Free        *
- * Software Foundation; Version 2.  This guarantees your right to use,     *
- * modify, and redistribute this software under certain conditions.  If    *
- * you wish to embed Nmap technology into proprietary software, we may be  *
- * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as  *
- * our remote OS fingerprinting database and code, service/version         *
- * detection system, and port scanning code.                               *
+ * The Nmap Security Scanner is (C) 1996-2008 Insecure.Com LLC. Nmap is    *
+ * also a registered trademark of Insecure.Com LLC.  This program is free  *
+ * software; you may redistribute and/or modify it under the terms of the  *
+ * GNU General Public License as published by the Free Software            *
+ * Foundation; Version 2 with the clarifications and exceptions described  *
+ * below.  This guarantees your right to use, modify, and redistribute     *
+ * this software under certain conditions.  If you wish to embed Nmap      *
+ * technology into proprietary software, we sell alternative licenses      *
+ * (contact sales@insecure.com).  Dozens of software vendors already       *
+ * license Nmap technology such as host discovery, port scanning, OS       *
+ * detection, and version detection.                                       *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -24,7 +23,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -38,7 +37,7 @@
  * These restrictions only apply when you actually redistribute Nmap.  For *
  * example, nothing stops you from writing and selling a proprietary       *
  * front-end to Nmap.  Just distribute it by itself, and point people to   *
- * http://www.insecure.org/nmap/ to download Nmap.                         *
+ * http://nmap.org to download Nmap.                                       *
  *                                                                         *
  * We don't consider these to be added restrictions on top of the GPL, but *
  * just a clarification of how we interpret "derived works" as it applies  *
@@ -50,16 +49,16 @@
  * If you have any questions about the GPL licensing restrictions on using *
  * Nmap in non-GPL works, we would be happy to help.  As mentioned above,  *
  * we also offer alternative license to integrate Nmap into proprietary    *
- * applications and appliances.  These contracts have been sold to many    *
- * security vendors, and generally include a perpetual license as well as  *
- * providing for priority support and updates as well as helping to fund   *
- * the continued development of Nmap technology.  Please email             *
+ * applications and appliances.  These contracts have been sold to dozens  *
+ * of software vendors, and generally include a perpetual license as well  *
+ * as providing for priority support and updates as well as helping to     *
+ * fund the continued development of Nmap technology.  Please email        *
  * sales@insecure.com for further information.                             *
  *                                                                         *
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -77,7 +76,7 @@
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
  * and add new features.  You are highly encouraged to send your changes   *
  * to fyodor@insecure.org for possible incorporation into the main         *
- * distribution.  By sending these changes to Fyodor or one the            *
+ * distribution.  By sending these changes to Fyodor or one of the         *
  * Insecure.Org development mailing lists, it is assumed that you are      *
  * offering Fyodor and Insecure.Com LLC the unlimited, non-exclusive right *
  * to reuse, modify, and relicense the code.  Nmap will always be          *
@@ -91,19 +90,22 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.cc 3355 2006-05-15 22:37:31Z fyodor $ */
+/* $Id: portlist.cc 9502 2008-08-15 03:30:34Z fyodor $ */
 
 
 #include "portlist.h"
 #include "nmap_error.h"
 #include "nmap.h"
 #include "NmapOps.h"
+#include "services.h"
+#include "nmap_rpc.h"
+#include "tcpip.h"
 
 using namespace std;
 
@@ -126,6 +128,7 @@ Port::Port() {
   serviceprobe_hostname = serviceprobe_ostype = serviceprobe_devicetype = NULL;
   serviceprobe_tunnel = SERVICE_TUNNEL_NONE;
   serviceprobe_fp = NULL;
+  state_reason_init(&reason);
 }
 
 Port::~Port() {
@@ -264,6 +267,23 @@ int Port::getServiceDeductions(struct serviceDeductions *sd) {
 // one is available and the user should submit it.  tunnel must be
 // SERVICE_TUNNEL_NULL (normal) or SERVICE_TUNNEL_SSL (means ssl was
 // detected and we tried to tunnel through it ).
+
+char* Port::cstringSanityCheck(const char* string, int len) {
+	char* result;
+  int slen;
+
+  if(!string)
+	  return NULL;
+
+  slen = strlen(string);
+  if (slen > len) slen = len;
+  result = (char *) safe_malloc(slen + 1);
+  memcpy(result, string, slen);
+  result[slen] = '\0';
+  replacenonprintable(result, slen, '.'); 
+  return result;
+}
+
 void Port::setServiceProbeResults(enum serviceprobestate sres, 
 				  const char *sname,	
 				  enum service_tunnel_type tunnel, 
@@ -272,91 +292,25 @@ void Port::setServiceProbeResults(enum serviceprobestate sres,
 				  const char *ostype, const char *devicetype,
 				  const char *fingerprint) {
 
-  int slen;
-  serviceprobe_results = sres;
-  unsigned char *p;
-  serviceprobe_tunnel = tunnel;
-  if (sname) serviceprobe_service = strdup(sname);
-  if (fingerprint) serviceprobe_fp = strdup(fingerprint);
+	serviceprobe_results = sres;
+	serviceprobe_tunnel = tunnel;
 
-  if (product) {
-    slen = strlen(product);
-    if (slen > 64) slen = 64;
-    serviceprobe_product = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_product, product, slen);
-    serviceprobe_product[slen] = '\0';
-    p = (unsigned char *) serviceprobe_product;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
+	if (sname) 
+		serviceprobe_service = strdup(sname);
+	else
+		sname = NULL;
 
-  if (version) {
-    slen = strlen(version);
-    if (slen > 64) slen = 64;
-    serviceprobe_version = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_version, version, slen);
-    serviceprobe_version[slen] = '\0';
-    p = (unsigned char *) serviceprobe_version;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
-  if (extrainfo) {
-    slen = strlen(extrainfo);
-    if (slen > 128) slen = 128;
-    serviceprobe_extrainfo = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_extrainfo, extrainfo, slen);
-    serviceprobe_extrainfo[slen] = '\0';
-    p = (unsigned char *) serviceprobe_extrainfo;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
-  if (hostname) {
-    slen = strlen(hostname);
-    if (slen > 64) slen = 64;
-    serviceprobe_hostname = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_hostname, hostname, slen);
-    serviceprobe_hostname[slen] = '\0';
-    p = (unsigned char *) serviceprobe_hostname;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-
-  if (ostype) {
-    slen = strlen(ostype);
-    if (slen > 64) slen = 64;
-    serviceprobe_ostype = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_ostype, ostype, slen);
-    serviceprobe_ostype[slen] = '\0';
-    p = (unsigned char *) serviceprobe_ostype;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
-  
-  if (devicetype) {
-    slen = strlen(devicetype);
-    if (slen > 64) slen = 64;
-    serviceprobe_devicetype = (char *) safe_malloc(slen + 1);
-    memcpy(serviceprobe_devicetype, devicetype, slen);
-    serviceprobe_devicetype[slen] = '\0';
-    p = (unsigned char *) serviceprobe_devicetype;
-    while(*p) {
-      if (!isprint((int)*p)) *p = '.';
-      p++;
-    }
-  }
+	if (fingerprint) 
+		serviceprobe_fp = strdup(fingerprint);
+	else 
+		serviceprobe_fp = NULL;
 
+	serviceprobe_product = cstringSanityCheck(product, 64);
+	serviceprobe_version = cstringSanityCheck(version, 64);
+	serviceprobe_extrainfo = cstringSanityCheck(extrainfo, 128);
+	serviceprobe_hostname = cstringSanityCheck(hostname, 64);
+	serviceprobe_ostype = cstringSanityCheck(ostype, 64);
+	serviceprobe_devicetype = cstringSanityCheck(devicetype, 64);
 }
 
 /* Sets the results of an RPC scan.  if rpc_status is not
@@ -382,11 +336,11 @@ void Port::setRPCProbeResults(int rpcs, unsigned long rpcp,
     serviceprobe_service = strdup(newsvc);
     serviceprobe_product = strdup(newsvc);
     if (rpc_lowver == rpc_highver)
-      snprintf(verbuf, sizeof(verbuf), "%i", rpc_lowver);
+      Snprintf(verbuf, sizeof(verbuf), "%i", rpc_lowver);
     else
-      snprintf(verbuf, sizeof(verbuf), "%i-%i", rpc_lowver, rpc_highver);
+      Snprintf(verbuf, sizeof(verbuf), "%i-%i", rpc_lowver, rpc_highver);
     serviceprobe_version = strdup(verbuf);
-    snprintf(verbuf, sizeof(verbuf), "rpc #%li", rpc_program);
+    Snprintf(verbuf, sizeof(verbuf), "rpc #%li", rpc_program);
     serviceprobe_extrainfo = strdup(verbuf);
   } else if (rpc_status == RPC_STATUS_UNKNOWN) {
     if (serviceprobe_service)
@@ -416,6 +370,7 @@ PortList::PortList() {
   }
 
   numports = 0;
+  numscriptresults = 0;
   idstr = NULL;
 }
 
@@ -448,7 +403,7 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
 
   if ((state == PORT_OPEN && o.verbose) || (o.debugging > 1)) {
     if (owner && *owner) {
-      snprintf(msg, sizeof(msg), " (owner: %s)", owner);
+      Snprintf(msg, sizeof(msg), " (owner: %s)", owner);
     } else msg[0] = '\0';
     
     log_write(LOG_STDOUT, "Discovered %s port %hu/%s%s%s\n",
@@ -462,7 +417,7 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
   if (state != PORT_OPEN && state != PORT_CLOSED && state != PORT_FILTERED &&
       state != PORT_UNFILTERED && state != PORT_OPENFILTERED && 
       state != PORT_CLOSEDFILTERED)
-    fatal("addPort: attempt to add port number %d with illegal state %d\n", portno, state);
+    fatal("%s: attempt to add port number %d with illegal state %d\n", __func__, portno, state);
 
   assert(protocol!=IPPROTO_IP || portno<256);
 
@@ -471,7 +426,7 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
     /* We must discount our statistics from the old values.  Also warn
        if a complete duplicate */
     if (o.debugging && current->state == state && (!owner || !*owner)) {
-      error("Duplicate port (%hu/%s)\n", portno, proto2ascii(protocol));
+      error("Duplicate port (%hu/%s)", portno, proto2ascii(protocol));
     } 
     state_counts_proto[proto][current->state]--;
   } else {
@@ -491,14 +446,16 @@ int PortList::addPort(u16 portno, u8 protocol, char *owner, int state) {
       free(current->owner);
     current->owner = strdup(owner);
   }
-  
+ 
+  if(state == PORT_FILTERED || state == PORT_OPENFILTERED)
+  	setStateReason(portno, protocol, ER_NORESPONSE, 0, 0); 
   return 0; /*success */
 }
 
 int PortList::removePort(u16 portno, u8 protocol) {
   Port *answer = NULL;
 
-  printf("Removed %d\n", portno);
+  log_write(LOG_PLAIN, "Removed %d\n", portno);
 
   answer = getPortEntry(portno, protocol);
   if (!answer)
@@ -532,7 +489,7 @@ void PortList::setIdStr(const char *id) {
   len = strlen(id);
   len += 5; // " on " + \0
   idstr = (char *) safe_malloc(len);
-  snprintf(idstr, len, " on %s", id);
+  Snprintf(idstr, len, " on %s", id);
 }
 
 
@@ -598,7 +555,7 @@ Port *PortList::getPortEntry(u16 portno, u8 protocol) {
 
   assert(protocol!=IPPROTO_IP || portno<256);
   if(port_map[proto]==NULL || port_list[proto]==NULL)
-    fatal("getPortEntry(%i,%i): you're trying to access uninitialized protocol", portno, protocol);
+    fatal("%s(%i,%i): you're trying to access uninitialized protocol", __func__, portno, protocol);
   mapped_pno = port_map[proto][portno];
 
   assert(mapped_pno < port_list_count[proto]);
@@ -606,7 +563,7 @@ Port *PortList::getPortEntry(u16 portno, u8 protocol) {
   
   /* The ugly hack: we allow only port 0 to be mapped to 0 position */
   if(mapped_pno==0 && portno!=0) {
-    error("WARNING: getPortEntry(%i,%i): this port was not mapped", portno, protocol);
+    error("WARNING: %s(%i,%i): this port was not mapped", __func__, portno, protocol);
     return(NULL);
   }else
     return(port_list[proto][mapped_pno]);
@@ -618,7 +575,7 @@ void PortList::setPortEntry(u16 portno, u8 protocol, Port *port) {
 
   assert(protocol!=IPPROTO_IP || portno<256);
   if(port_map[proto]==NULL || port_list[proto]==NULL)
-    fatal("setPortEntry(%i,%i): you're trying to access uninitialized protocol", portno, protocol);
+    fatal("%s(%i,%i): you're trying to access uninitialized protocol", __func__, portno, protocol);
   mapped_pno = port_map[proto][portno];
 
   assert(mapped_pno < port_list_count[proto]);
@@ -626,13 +583,24 @@ void PortList::setPortEntry(u16 portno, u8 protocol, Port *port) {
   
   /* The ugly hack: we allow only port 0 to be mapped to 0 position */
   if(mapped_pno==0 && portno!=0) {
-    error("WARNING: setPortEntry(%i,%i): this port was not mapped", portno, protocol);
+    error("WARNING: %s(%i,%i): this port was not mapped", __func__, portno, protocol);
     return;
   }
   
   port_list[proto][mapped_pno] = port;
 }
 
+/* Just free memory used by PortList::port_map[]. Should be done somewhere 
+ * before closing nmap. */
+void PortList::freePortMap(){
+  int proto;
+  for(proto=0; proto < PORTLIST_PROTO_MAX; proto++)
+    if(port_map[proto]){
+      free(port_map[proto]);
+      port_map[proto] = NULL;
+  }
+}
+  
 
 u16 *PortList::port_map[PORTLIST_PROTO_MAX];
 int PortList::port_list_count[PORTLIST_PROTO_MAX];
@@ -647,7 +615,7 @@ void PortList::initializePortMap(int protocol, u16 *ports, int portcount) {
   int proto = INPROTO2PORTLISTPROTO(protocol);
   
   if(port_map[proto]!=NULL)
-    fatal("initializePortMap: portmap for protocol %i already initialized", protocol);
+    fatal("%s: portmap for protocol %i already initialized", __func__, protocol);
 
   assert(port_list_count[proto]==0);
   
@@ -724,11 +692,24 @@ bool PortList::isIgnoredState(int state) {
       state == PORT_FRESH)
     return false; /* Cannot be ignored */
 
+  if (state == PORT_OPENFILTERED && (o.verbose > 2 || o.debugging > 2))
+    return false;
+
+  /* If openonly, we always ignore states that don't at least have open
+     as a possibility. */
+  if (o.openOnly() && state != PORT_OPENFILTERED && state != PORT_UNFILTERED 
+      && getStateCounts(state) > 0)
+    return true;
+
   int max_per_state = 25; // Ignore states with more ports than this
   /* We will show more ports when verbosity is requested */
-  if (o.verbose || o.debugging)
-    max_per_state *= (o.verbose + 50 * o.debugging);
-  
+  if (o.verbose || o.debugging) {
+    if (o.ipprotscan)
+      max_per_state *= (o.verbose + 3 * o.debugging);
+    else
+      max_per_state *= (o.verbose + 20 * o.debugging);
+  }
+
   if (getStateCounts(state) > max_per_state)
     return true;
 
@@ -754,6 +735,22 @@ int PortList::numIgnoredPorts() {
   return numports;
 }
 
+int PortList::setStateReason(u16 portno, u8 proto, reason_t reason, u8 ttl, u32 ip_addr) {
+    Port *answer = NULL;
+
+    if(!(answer = getPortEntry(portno, proto))) 
+       	return -1;
+    if(reason > ER_MAX)
+        return -1;
+
+    /* set new reason and increment its count */
+    answer->reason.reason_id = reason;
+    answer->reason.ip_addr.s_addr = ip_addr;
+	answer->reason.ttl = ttl;
+    answer->reason.state = answer->state;
+    setPortEntry(portno, proto, answer);
+    return 0;
+}
 
 // Move some popular TCP ports to the beginning of the portlist, because
 // that can speed up certain scans.  You should have already done any port
