@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -60,7 +60,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -92,13 +92,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: service_scan.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: service_scan.cc 8886 2008-07-14 19:52:31Z kris $ */
 
 
 #include "service_scan.h"
@@ -132,10 +132,6 @@
 namespace std {};
 using namespace std;
 
-// Because this file uses assert()s for some security checking, we can't
-// have anyone turning off debugging.
-#undef NDEBUG
-
 extern NmapOps o;
 
 // Details on a particular service (open port) we are trying to match
@@ -216,7 +212,7 @@ private:
   // written if there is enough space.  Otherwise it exits.
   void addServiceChar(char c, int wrapat);
   // Like addServiceChar, but for a whole zero-terminated string
-  void addServiceString(char *s, int wrapat);
+  void addServiceString(const char *s, int wrapat);
   vector<ServiceProbe *>::iterator current_probe;
   u8 *currentresp;
   int currentresplen;
@@ -1074,10 +1070,10 @@ void parse_nmap_service_probe_file(AllProbes *AP, char *filename) {
       continue;
   
     if (strncmp(line, "Exclude ", 8) == 0) {
-      if (AP->excludedports != NULL)
+      if (AP->excluded_seen)
         fatal("Only 1 Exclude directive is allowed in the nmap-service-probes file");
-
-      AP->excludedports = getpts(line+8);
+      getpts(line+8, &AP->excludedports);
+      AP->excluded_seen = true;
       continue;
     }
   
@@ -1195,7 +1191,8 @@ const struct MatchDetails *ServiceProbe::testMatch(const u8 *buf, int buflen) {
 
 AllProbes::AllProbes() {
   nullProbe = NULL;
-  excludedports = NULL;
+  excluded_seen = false;
+  memset(&excludedports, 0, sizeof(excludedports));
 }
 
 AllProbes::~AllProbes() {
@@ -1207,7 +1204,7 @@ AllProbes::~AllProbes() {
   }
   if(nullProbe)
     delete nullProbe;
-  free_scan_lists(excludedports);
+  free_scan_lists(&excludedports);
 }
 
   // Tries to find the probe in this AllProbes class which have the
@@ -1239,14 +1236,14 @@ int AllProbes::isExcluded(unsigned short port, int proto) {
   unsigned short *p=NULL;
   int count=-1,i;
 
-  if (!excludedports) return 0;
+  if (!excluded_seen) return 0;
 
   if (proto == IPPROTO_TCP) {
-    p = excludedports->tcp_ports;
-    count = excludedports->tcp_count;
+    p = excludedports.tcp_ports;
+    count = excludedports.tcp_count;
   } else if (proto == IPPROTO_UDP) {
-    p = excludedports->udp_ports;
-    count = excludedports->udp_count;
+    p = excludedports.udp_ports;
+    count = excludedports.udp_count;
   } else {
     fatal("Bad proto number (%d) specified in %s", proto, __func__);
   }
@@ -1354,7 +1351,7 @@ ServiceNFO::~ServiceNFO() {
   // Adds a character to servicefp.  Takes care of word wrapping if
   // necessary at the given (wrapat) column.  Chars will only be
   // written if there is enough space.  Otherwise it exits.
-void ServiceNFO::addServiceChar(char c, int wrapat) {
+void ServiceNFO::addServiceChar(const char c, int wrapat) {
 
   if (servicefpalloc - servicefplen < 6)
     fatal("%s - out of space for servicefp", __func__);
@@ -1369,7 +1366,7 @@ void ServiceNFO::addServiceChar(char c, int wrapat) {
 }
 
 // Like addServiceChar, but for a whole zero-terminated string
-void ServiceNFO::addServiceString(char *s, int wrapat) {
+void ServiceNFO::addServiceString(const char *s, int wrapat) {
   while(*s) 
     addServiceChar(*s++, wrapat);
 }
@@ -1769,6 +1766,12 @@ static void startNextProbe(nsock_pool nsp, nsock_iod nsi, ServiceGroup *SG,
 	if ((svc->niod = nsi_new(nsp, svc)) == NULL) {
 	  fatal("Failed to allocate Nsock I/O descriptor in %s()", __func__);
 	}
+	if (o.spoofsource) {
+	  o.SourceSockAddr(&ss, &ss_len);
+	  nsi_set_localaddr(svc->niod, &ss, ss_len);
+	}
+	if (o.ipoptionslen)
+	  nsi_set_ipoptions(svc->niod, o.ipoptions, o.ipoptionslen);
 	svc->target->TargetSockAddr(&ss, &ss_len);
 	if (svc->tunnel == SERVICE_TUNNEL_NONE) {
 	  nsock_connect_tcp(nsp, svc->niod, servicescan_connect_handler, 
@@ -1967,6 +1970,12 @@ static int launchSomeServiceProbes(nsock_pool nsp, ServiceGroup *SG) {
     if (o.debugging > 1) {
       log_write(LOG_PLAIN, "Starting probes against new service: %s:%hi (%s)\n", svc->target->targetipstr(), svc->portno, proto2ascii(svc->proto));
     }
+    if (o.spoofsource) {
+      o.SourceSockAddr(&ss, &ss_len);
+      nsi_set_localaddr(svc->niod, &ss, ss_len);
+    }
+    if (o.ipoptionslen)
+      nsi_set_ipoptions(svc->niod, o.ipoptions, o.ipoptionslen);
     svc->target->TargetSockAddr(&ss, &ss_len);
     if (svc->proto == IPPROTO_TCP)
       nsock_connect_tcp(nsp, svc->niod, servicescan_connect_handler, 
@@ -2136,7 +2145,7 @@ static void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *myda
 	// example, if we read more data for the same probe response
 	// it will probably still match.
       } else {
-	if (o.debugging > 1)
+	if (o.debugging > 1) {
 	  if (MD->product || MD->version || MD->info)
 	    log_write(LOG_PLAIN, "Service scan match (Probe %s matched with %s): %s:%hi is %s%s.  Version: |%s|%s|%s|\n",
                       probe->getName(), (*probe->fallbacks[fallbackDepth]).getName(),
@@ -2148,6 +2157,7 @@ static void servicescan_read_handler(nsock_pool nsp, nsock_event nse, void *myda
                       (MD->isSoft)? "soft" : "hard",
                       probe->getName(), (*probe->fallbacks[fallbackDepth]).getName(),
 		      svc->target->NameIP(), svc->portno, (svc->tunnel == SERVICE_TUNNEL_SSL)? "SSL/" : "", MD->serviceName);
+	}
 	svc->probe_matched = MD->serviceName;
 	if (MD->product)
 	  Strncpy(svc->product_matched, MD->product, sizeof(svc->product_matched));
