@@ -26,7 +26,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -61,7 +61,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -93,13 +93,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: timing.cc 9525 2008-08-16 01:05:11Z david $ */
 
 #include "timing.h"
 #include "NmapOps.h"
@@ -238,7 +238,208 @@ void enforce_scan_delay(struct timeval *tv) {
   return;    
 }
 
-ScanProgressMeter::ScanProgressMeter(char *stypestr) {
+/* current_rate_history defines how far back (in seconds) we look when
+   calculating the current rate. */
+RateMeter::RateMeter(double current_rate_history) {
+  this->current_rate_history = current_rate_history;
+  start_tv.tv_sec = 0;
+  start_tv.tv_usec = 0;
+  stop_tv.tv_sec = 0;
+  stop_tv.tv_usec = 0;
+  last_update_tv.tv_sec = 0;
+  last_update_tv.tv_usec = 0;
+  total = 0.0;
+  current_rate = 0.0;
+  assert(!isSet(&start_tv));
+  assert(!isSet(&stop_tv));
+}
+
+void RateMeter::start(const struct timeval *now) {
+  assert(!isSet(&start_tv));
+  assert(!isSet(&stop_tv));
+  if (now == NULL)
+    gettimeofday(&start_tv, NULL);
+  else
+    start_tv = *now;
+}
+
+void RateMeter::stop(const struct timeval *now) {
+  assert(isSet(&start_tv));
+  assert(!isSet(&stop_tv));
+  if (now == NULL)
+    gettimeofday(&stop_tv, NULL);
+  else
+    stop_tv = *now;
+}
+
+/* Update the rates to reflect the given amount added to the total at the time
+   now. If now is NULL, get the current time with gettimeofday. */
+void RateMeter::update(double amount, const struct timeval *now) {
+  struct timeval tv;
+  static int clockwarn = 0;
+  double diff;
+  double interval;
+  double count;
+
+  assert(isSet(&start_tv));
+  assert(!isSet(&stop_tv));
+
+  /* Update the total. */
+  total += amount;
+
+  if (now == NULL) {
+    gettimeofday(&tv, NULL);
+    now = &tv;
+  }
+  if (!isSet(&last_update_tv))
+    last_update_tv = start_tv;
+
+  /* Calculate the approximate moving average of how much was recorded in the
+     last current_rate_history seconds. This average is what is returned as the
+     "current" rate. */
+
+  /* How long since the last update? */
+  diff = TIMEVAL_SUBTRACT(*now, last_update_tv) / 1000000.0;
+
+  /* On my SMP Linux 2.6.20 system, I'm getting very occasional values
+     like "now=1214173867.8027; last_update_tv=1214173867.8065".
+     Unless I'm missing something, I think my gettimeofday may have
+     decreased by 38 microseconds.  Perhaps due to SMP and the old
+     kernel.  Anyway, I think I'll just treat decreases of up to 1ms
+     as zero -Fyodor 
+     Updated to allow up to 5ms due to crash:
+     RateMeter::update: negative time delta; now=1217210189.144224; last_update_tv=1217210189.148486 */
+  if (diff < 0 && diff > -0.005) {
+    if (!clockwarn) {
+      error("Warning: RateMeter::update: negative time delta; now=%lu.%lu; last_update_tv=%lu.%lu", (unsigned long) now->tv_sec, (unsigned long) now->tv_usec, (unsigned long) last_update_tv.tv_sec, (unsigned long) last_update_tv.tv_usec);
+      clockwarn = 1;
+    }
+    diff = 0.0;
+  }
+  if (diff < 0.0)
+    fatal("RateMeter::update: negative time delta; now=%lu.%lu; last_update_tv=%lu.%lu", (unsigned long) now->tv_sec, (unsigned long) now->tv_usec, (unsigned long) last_update_tv.tv_sec, (unsigned long) last_update_tv.tv_usec);
+
+  /* Find out how far back in time to look. We want to look back
+     current_rate_history seconds, or to when the last update occurred,
+     whichever is longer. However, we never look past the start. */
+  struct timeval tmp;
+  /* Find the time current_rate_history seconds after the start. That's our
+     threshold for deciding how far back to look. */
+  TIMEVAL_ADD(tmp, start_tv, (time_t) (current_rate_history * 1000000.0));
+  if (TIMEVAL_AFTER(*now, tmp))
+    interval = MAX(current_rate_history, diff);
+  else
+    interval = TIMEVAL_SUBTRACT(*now, start_tv) / 1000000.0;
+  assert(diff <= interval);
+  /* If we record an amount in the very same instant that the timer is started,
+     there's no way to calculate meaningful rates. Ignore it. */
+  if (interval == 0.0)
+    return;
+
+  /* To calculate the approximate average of the rate over the last
+     interval seconds, we assume that the rate was constant over that interval.
+     We calculate how much would have been received in that interval, ignoring
+     the first diff seconds' worth:
+       (interval - diff) * current_rate.
+     Then we add how much was received in the most recent diff seconds. Divide
+     by the width of the interval to get the average. */
+  count = (interval - diff) * current_rate + amount;
+  current_rate = count / interval;
+
+  last_update_tv = *now;
+}
+
+double RateMeter::getOverallRate(const struct timeval *now) const {
+  return total / elapsedTime(now);
+}
+
+/* Get the "current" rate (actually a moving average of the last
+   current_rate_history seconds). If update is true (its default value), lower
+   the rate to account for the time since the last record. */
+double RateMeter::getCurrentRate(const struct timeval *now, bool update) {
+  if (update)
+    this->update(0.0, now);
+
+  return current_rate;
+}
+
+double RateMeter::getTotal(void) const {
+  return total;
+}
+
+/* Get the number of seconds the meter has been running: if it has been stopped,
+   the amount of time between start and stop, or if it is still running, the
+   amount of time between start and now. */
+double RateMeter::elapsedTime(const struct timeval *now) const {
+  struct timeval tv;
+  const struct timeval *end_tv;
+
+  assert(isSet(&start_tv));
+
+  if (isSet(&stop_tv)) {
+    end_tv = &stop_tv;
+  } else if (now == NULL) {
+    gettimeofday(&tv, NULL);
+    end_tv = &tv;
+  } else {
+    end_tv = now;
+  }
+
+  return TIMEVAL_SUBTRACT(*end_tv, start_tv) / 1000000.0;
+}
+
+/* Returns true if tv has been initialized; i.e., its members are not all
+   zero. */
+bool RateMeter::isSet(const struct timeval *tv) {
+  return tv->tv_sec != 0 || tv->tv_usec != 0;
+}
+
+PacketRateMeter::PacketRateMeter(double current_rate_history) {
+  packet_rate_meter = RateMeter(current_rate_history);
+  byte_rate_meter = RateMeter(current_rate_history);
+}
+
+void PacketRateMeter::start(const struct timeval *now) {
+  packet_rate_meter.start(now);
+  byte_rate_meter.start(now);
+}
+
+void PacketRateMeter::stop(const struct timeval *now) {
+  packet_rate_meter.stop(now);
+  byte_rate_meter.stop(now);
+}
+
+/* Record one packet of length len. */
+void PacketRateMeter::update(u32 len, const struct timeval *now) {
+  packet_rate_meter.update(1, now);
+  byte_rate_meter.update(len, now);
+}
+
+double PacketRateMeter::getOverallPacketRate(const struct timeval *now) const {
+  return packet_rate_meter.getOverallRate(now);
+}
+
+double PacketRateMeter::getCurrentPacketRate(const struct timeval *now, bool update) {
+  return packet_rate_meter.getCurrentRate(now, update);
+}
+
+double PacketRateMeter::getOverallByteRate(const struct timeval *now) const {
+  return byte_rate_meter.getOverallRate(now);
+}
+
+double PacketRateMeter::getCurrentByteRate(const struct timeval *now, bool update) {
+  return byte_rate_meter.getCurrentRate(now, update);
+}
+
+unsigned long long PacketRateMeter::getNumPackets(void) const {
+  return (unsigned long long) packet_rate_meter.getTotal();
+}
+
+unsigned long long PacketRateMeter::getNumBytes(void) const {
+  return (unsigned long long) byte_rate_meter.getTotal();
+}
+
+ScanProgressMeter::ScanProgressMeter(const char *stypestr) {
   scantypestr = strdup(stypestr);
   gettimeofday(&begin, NULL);
   last_print_test = begin;
