@@ -26,7 +26,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -61,7 +61,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -93,13 +93,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: timing.h 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: timing.h 9525 2008-08-16 01:05:11Z david $ */
 
 #ifndef NMAP_TIMING_H
 #define NMAP_TIMING_H
@@ -123,16 +123,67 @@ void adjust_timeouts2(const struct timeval *sent,
    response.  We update our RTT averages, etc. */
 void adjust_timeouts(struct timeval sent, struct timeout_info *to);
 
+#define DEFAULT_CURRENT_RATE_HISTORY 5.0
 
 /* Sleeps if necessary to ensure that it isn't called twice within less
    time than o.send_delay.  If it is passed a non-null tv, the POST-SLEEP
    time is recorded in it */
 void enforce_scan_delay(struct timeval *tv);
 
+/* This class measures current and lifetime average rates for some quantity. */
+class RateMeter {
+  public:
+    RateMeter(double current_rate_history = DEFAULT_CURRENT_RATE_HISTORY);
+
+    void start(const struct timeval *now = NULL);
+    void stop(const struct timeval *now = NULL);
+    void update(double amount, const struct timeval *now = NULL);
+    double getOverallRate(const struct timeval *now = NULL) const;
+    double getCurrentRate(const struct timeval *now = NULL, bool update = true);
+    double getTotal(void) const;
+
+  private:
+    /* How many seconds to look back when calculating the "current" rates. */
+    double current_rate_history;
+
+    /* When this meter started recording. */
+    struct timeval start_tv;
+    /* When this meter stopped recording. */
+    struct timeval stop_tv;
+    /* The last time the current sample rates were updated. */
+    struct timeval last_update_tv;
+
+    double total;
+    double current_rate;
+
+    double elapsedTime(const struct timeval *now = NULL) const;
+    static bool isSet(const struct timeval *tv);
+};
+
+/* A specialization of RateMeter that measures packet and byte rates. */
+class PacketRateMeter {
+  public:
+    PacketRateMeter(double current_rate_history = DEFAULT_CURRENT_RATE_HISTORY);
+
+    void start(const struct timeval *now = NULL);
+    void stop(const struct timeval *now = NULL);
+    void update(u32 len, const struct timeval *now = NULL);
+    double getOverallPacketRate(const struct timeval *now = NULL) const;
+    double getCurrentPacketRate(const struct timeval *now = NULL, bool update = true);
+    double getOverallByteRate(const struct timeval *now = NULL) const;
+    double getCurrentByteRate(const struct timeval *now = NULL, bool update =true);
+    unsigned long long getNumPackets(void) const;
+    unsigned long long getNumBytes(void) const;
+
+  private:
+    RateMeter packet_rate_meter;
+    RateMeter byte_rate_meter;
+};
+
 class ScanProgressMeter {
  public:
   /* A COPY of stypestr is made and saved for when stats are printed */
-  ScanProgressMeter(char *stypestr);
+  ScanProgressMeter(const char *stypestr);
   ~ScanProgressMeter();
 /* Decides whether a timing report is likely to even be
    printed.  There are stringent limitations on how often they are
