@@ -24,7 +24,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -59,7 +59,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -91,13 +91,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: utils.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: utils.cc 8924 2008-07-15 20:06:05Z kris $ */
 
 #include "nmap.h"
 #include "utils.h"
@@ -449,28 +449,6 @@ void arg_parse_free(char **argv) {
   free(argv);
 }
 
-/* Converts an Nmap time specification string into milliseconds.  If
-   the string is a plain non-negative number, it is considered to
-   already be in milliseconds and is returned.  If it is a number
-   followed by 's' (for seconds), 'm' (minutes), or 'h' (hours), the
-   number is converted to milliseconds and returned.  If Nmap cannot
-   parse the string, it is returned instead. */
-long tval2msecs(char *tspec) {
-  long l;
-  char *endptr = NULL;
-  l = strtol(tspec, &endptr, 10);
-  if (l < 0 || !endptr) return -1;
-  if (*endptr == '\0') return l;
-  if (*endptr == 's' || *endptr == 'S') return l * 1000;
-  if ((*endptr == 'm' || *endptr == 'M')) {
-    if (*(endptr + 1) == 's' || *(endptr + 1) == 'S') 
-      return l;
-    return l * 60000;
-  }
-  if (*endptr == 'h' || *endptr == 'H') return l * 3600000;
-  return -1;
-}
-
 // A simple function to form a character from 2 hex digits in ASCII form
 static unsigned char hex2char(unsigned char a, unsigned char b)
 {
@@ -704,7 +682,7 @@ void bintohexstr(char *buf, int buflen, char *src, int srclen){
       bp += Snprintf(buf+bp, buflen-bp,"\n");
 }
 
-static inline char* STRAPP(char *fmt, ...) {
+static inline char* STRAPP(const char *fmt, ...) {
   static char buf[256];
   static int bp;
   int left = (int)sizeof(buf)-bp;
