@@ -101,7 +101,14 @@ class ControlAction(BWExpander):
                                self.__change_pointer,
                                POINTER_JUMP_TO)
 
-        self.__info = gtk.RadioToolButton(self.__jump_to, gtk.STOCK_INFO)
+        try:
+            # gtk.STOCK_INFO is available only in PyGTK 2.8 and later.
+            info_icon = gtk.STOCK_INFO
+        except AttributeError:
+            self.__info = gtk.RadioToolButton(self.__jump_to, None)
+            self.__info.set_label(_("Info"))
+        else:
+            self.__info = gtk.RadioToolButton(self.__jump_to, info_icon)
         self.__info.set_tooltip(self.__tooltips, 'Show information')
         self.__info.connect('toggled',
                             self.__change_pointer,
