@@ -407,86 +407,9 @@ class SystemPage(BWScrolledWindow):
         self.__general_frame.bw_add(self.__general)
         self.__general_frame.set_expanded(True)
 
-        # sequences information widgets
-        self.__sequences = BWTable(5, 3)
-
         sequences = self.__node.get_info('sequences')
-
         if len(sequences) > 0:
-
-            self.__sequences.attach(BWSectionLabel('Class'), 1, 2, 0, 1)
-            self.__sequences.attach(BWSectionLabel('Values'), 2, 3, 0, 1)
-
-            self.__sequences.attach(BWSectionLabel('TCP *'), 0, 1, 1, 2)
-            self.__sequences.attach(BWSectionLabel('IP ID'), 0, 1, 2, 3)
-            self.__sequences.attach(BWSectionLabel('TCP Timestamp'),
-                                    0,
-                                    1,
-                                    3,
-                                    4)
-
-            # tcp sequence values
-            tcp = sequences['tcp']
-
-            tcp_class = BWLabel(tcp['class'])
-            tcp_class.set_selectable(True)
-
-            self.__sequences.attach(tcp_class, 1, 2, 1, 2)
-
-            tcp_values = gtk.combo_box_entry_new_text()
-
-            for value in tcp['values']:
-                tcp_values.append_text(value)
-
-            tcp_values.set_active(0)
-
-            self.__sequences.attach(tcp_values, 2, 3, 1, 2)
-
-            tcp_note = BWLabel()
-            tcp_note.set_selectable(True)
-            tcp_note.set_line_wrap(False)
-            tcp_note.set_alignment(1.0, 0.5)
-            tcp_note.set_markup(TCP_SEQ_NOTE % (tcp['index'], tcp['difficulty']))
-
-            self.__sequences.attach(tcp_note, 0, 3, 4, 5)
-
-            # ip id sequence values
-            ip_id = sequences['ip_id']
-
-            ip_id_class = BWLabel(ip_id['class'])
-            ip_id_class.set_selectable(True)
-
-            self.__sequences.attach(ip_id_class, 1, 2, 2, 3)
-
-            ip_id_values = gtk.combo_box_entry_new_text()
-
-            for value in ip_id['values']:
-                ip_id_values.append_text(value)
-
-            ip_id_values.set_active(0)
-
-            self.__sequences.attach(ip_id_values, 2, 3, 2, 3)
-
-            # tcp sequence values
-            tcp_ts = sequences['tcp_ts']
-
-            tcp_ts_class = BWLabel(tcp_ts['class'])
-            tcp_ts_class.set_selectable(True)
-
-            self.__sequences.attach(tcp_ts_class, 1, 2, 3, 4)
-
-            if tcp_ts['values'] != None:
-
-                tcp_ts_values = gtk.combo_box_entry_new_text()
-
-                for value in tcp_ts['values']:
-                    tcp_ts_values.append_text(value)
-
-                tcp_ts_values.set_active(0)
-
-                self.__sequences.attach(tcp_ts_values, 2, 3, 3, 4)
-
-            self.__sequences_frame.bw_add(self.__sequences)
+            self.__sequences_frame.bw_add(self.__create_sequences_widget(sequences))
 
         # operating system information widgets
         self.__os = gtk.Notebook()
@@ -629,6 +552,81 @@ class SystemPage(BWScrolledWindow):
 
         self.add_with_viewport(self.__vbox)
 
+    def __create_sequences_widget(self, sequences):
+        """Return a widget representing various OS detection sequences. The
+        sequences argument is a dict with zero or more of the keys 'tcp',
+        'ip_id', and 'tcp_ts'."""
+        # sequences information widgets
+        table = BWTable(5, 3)
+
+        table.attach(BWSectionLabel('Class'), 1, 2, 0, 1)
+        table.attach(BWSectionLabel('Values'), 2, 3, 0, 1)
+
+        table.attach(BWSectionLabel('TCP *'), 0, 1, 1, 2)
+        table.attach(BWSectionLabel('IP ID'), 0, 1, 2, 3)
+        table.attach(BWSectionLabel('TCP Timestamp'), 0, 1, 3, 4)
+
+        tcp = sequences.get('tcp')
+        if tcp is not None:
+            tcp_class = BWLabel(tcp['class'])
+            tcp_class.set_selectable(True)
+
+            table.attach(tcp_class, 1, 2, 1, 2)
+
+            tcp_values = gtk.combo_box_entry_new_text()
+
+            for value in tcp['values']:
+                tcp_values.append_text(value)
+
+            tcp_values.set_active(0)
+
+            table.attach(tcp_values, 2, 3, 1, 2)
+
+            tcp_note = BWLabel()
+            tcp_note.set_selectable(True)
+            tcp_note.set_line_wrap(False)
+            tcp_note.set_alignment(1.0, 0.5)
+            tcp_note.set_markup(TCP_SEQ_NOTE % (tcp['index'], tcp['difficulty']))
+
+            table.attach(tcp_note, 0, 3, 4, 5)
+
+        ip_id = sequences.get('ip_id')
+        if ip_id is not None:
+            ip_id_class = BWLabel(ip_id['class'])
+            ip_id_class.set_selectable(True)
+
+            table.attach(ip_id_class, 1, 2, 2, 3)
+
+            ip_id_values = gtk.combo_box_entry_new_text()
+
+            for value in ip_id['values']:
+                ip_id_values.append_text(value)
+
+            ip_id_values.set_active(0)
+
+            table.attach(ip_id_values, 2, 3, 2, 3)
+
+        tcp_ts = sequences.get('tcp_ts')
+        if tcp_ts is not None:
+            tcp_ts_class = BWLabel(tcp_ts['class'])
+            tcp_ts_class.set_selectable(True)
+
+            table.attach(tcp_ts_class, 1, 2, 3, 4)
+
+            if tcp_ts['values'] != None:
+
+                tcp_ts_values = gtk.combo_box_entry_new_text()
+
+                for value in tcp_ts['values']:
+                    tcp_ts_values.append_text(value)
+
+                tcp_ts_values.set_active(0)
+
+                table.attach(tcp_ts_values, 2, 3, 3, 4)
+
+        return table
+
+
 
 
 class TraceroutePage(BWVBox):
@@ -648,7 +646,11 @@ class TraceroutePage(BWVBox):
     def __create_widgets(self):
         """
         """
-        if self.__node.get_info('trace') == None:
+        trace = self.__node.get_info('trace')
+        hops = None
+        if trace is not None:
+            hops = trace.get("hops")
+        if hops is None or len(hops) == 0:
 
             self.__trace_label = gtk.Label(NO_TRACE_TEXT)
             self.pack_start(self.__trace_label, True, True)
