@@ -306,6 +306,8 @@ class RadialNet(gtk.DrawingArea):
     def update_layout(self):
         """
         """
+        if self.__graph is None:
+            return
         self.__animating = True
         self.__calc_interpolation(self.__graph.get_main_node())
         self.__livens_up()
@@ -1574,6 +1576,8 @@ class RadialNet(gtk.DrawingArea):
         """
         """
         nodes = list()
+        if self.__graph is None:
+            return nodes
 
         for node in self.__graph.get_nodes():
 
