@@ -420,7 +420,7 @@ class my_uninstall(Command):
 # example py2exe needs special handling.) These arguments are common between all
 # the operations.
 COMMON_SETUP_ARGS = {
-    'name': str(APP_NAME),
+    'name': APP_NAME,
     'license': 'GNU GPL (version 2 or later)',
     'url': APP_WEB_SITE,
     'download_url': APP_DOWNLOAD_SITE,
@@ -428,14 +428,13 @@ COMMON_SETUP_ARGS = {
     'author_email': 'py.adriano@gmail.com, cleber@globalred.com.br',
     'maintainer': 'Adriano Monteiro',
     'maintainer_email': 'py.adriano@gmail.com',
-    'description': """\
-%s is the %s frontend.""" % (APP_DISPLAY_NAME, NMAP_DISPLAY_NAME),
+    'description': "%s frontend and results viewer" % APP_DISPLAY_NAME,
     'long_description': """\
 %s is an %s frontend \
 that is really useful for advanced users and easy to be used by newbies.""" \
 % (APP_DISPLAY_NAME, NMAP_DISPLAY_NAME),
     'version': VERSION,
-    'scripts': [str(APP_NAME)],
+    'scripts': [APP_NAME],
     'packages': ['zenmapCore', 'zenmapGUI', 'zenmapGUI.higwidgets',
                  'radialnet', 'radialnet.bestwidgets', 'radialnet.core', 'radialnet.gui', 'radialnet.util'],
     'data_files': data_files,
@@ -451,8 +450,8 @@ if 'py2exe' in sys.argv:
 
     WINDOWS_SETUP_ARGS = {
         'zipfile': None,
-        'name': str(APP_NAME),
-        'windows': [{"script": str(APP_NAME),
+        'name': APP_NAME,
+        'windows': [{"script": APP_NAME,
                      "icon_resources": [(1, "install_scripts/windows/nmap-eye.ico")]}],
         'options': {"py2exe": {
             "compressed": 1,
@@ -483,7 +482,7 @@ elif 'py2app' in sys.argv:
     shutil.copyfile(APP_NAME, extended_app_name)
 
     MACOSX_SETUP_ARGS = {
-        'app': [str(extended_app_name)],
+        'app': [extended_app_name],
         'options': {"py2app": {
             "packages": ["gobject", "gtk", "cairo"],
             "includes": ["atk", "pango", "pangocairo"],
