@@ -22,20 +22,18 @@
 import gtk
 import os.path
 
-from higwidgets.higwindows import HIGWindow
-from higwidgets.higboxes import HIGVBox, HIGHBox, hig_box_space_holder
-from higwidgets.higlabels import HIGEntryLabel
-from higwidgets.higdialogs import HIGAlertDialog
-from higwidgets.higtables import HIGTable
+from zenmapGUI.higwidgets.higwindows import HIGWindow
+from zenmapGUI.higwidgets.higboxes import HIGVBox, HIGHBox, hig_box_space_holder
+from zenmapGUI.higwidgets.higlabels import HIGEntryLabel
+from zenmapGUI.higwidgets.higdialogs import HIGAlertDialog
+from zenmapGUI.higwidgets.higtables import HIGTable
 
 from zenmapGUI.OptionBuilder import *
 from zenmapGUI.ProfileEditor import *
 
 from zenmapCore.Name import NMAP_DISPLAY_NAME
 from zenmapCore.Paths import Path
-from zenmapCore.WizardConf import wizard_file
 from zenmapCore.TargetList import target_list
-from zenmapCore.NmapCommand import *
 from zenmapCore.UmitConf import Profile, CommandProfile
 from zenmapCore.I18N import _
 
@@ -55,7 +53,8 @@ class Wizard(HIGWindow):
         
         self.profile = CommandProfile()
         self.constructor = CommandConstructor()
-        self.options = OptionBuilder(wizard_file, self.constructor, self.update_command)
+        self.profilehelp = ProfileHelp()
+        self.options = OptionBuilder(Path.wizard, self.constructor, self.update_command, self.profilehelp)
         
         self.target = '<target>'
         
@@ -87,7 +86,7 @@ class Wizard(HIGWindow):
         self.set_title(_("%s command constructor wizard" % NMAP_DISPLAY_NAME))
         
         self.main_vbox._pack_expand_fill(self.directions['Start'])
-        self.set_notebook(None)
+        self.set_scan_interface(None)
         
         self.update_command()
 
@@ -120,8 +119,8 @@ class Wizard(HIGWindow):
         
         return vbox
 
-    def set_notebook(self, notebook):
-        self.notebook = notebook
+    def set_scan_interface(self, interface):
+        self.scan_interface = interface
 
     def __create_wizard_widgets(self):
         self.wizard_title = HIGEntryLabel("")
@@ -184,8 +183,8 @@ class Wizard(HIGWindow):
             self.directions['Start'].hide()
             self.directions['Choose'].show_all()
         else:
-            p = ProfileEditor()
-            p.set_notebook(self.notebook)
+            p = ProfileEditor(deletable = False)
+            p.set_scan_interface(self.scan_interface)
             p.show_all()
             
             self.close_wizard()
@@ -294,40 +293,26 @@ for this profile.'))
         if self.directions['Choose'].profile_radio.get_active():
             profile_name = self.directions['Profile'].profile_entry.get_text()
             
-            hint = self.directions['Profile'].hint_entry.get_text()
-            
             buffer = self.directions['Profile'].description_text.get_buffer()
             description = buffer.get_text(buffer.get_start_iter(),\
                                           buffer.get_end_iter())
             
-            buffer = self.directions['Profile'].annotation_text.get_buffer()
-            annotation = buffer.get_text(buffer.get_start_iter(),\
-                                          buffer.get_end_iter())
-            
             self.profile.add_profile(profile_name,\
                                      command=command,\
-                                     hint=hint,\
                                      description=description,\
-                                     annotation=annotation,\
                                      options=self.constructor.get_options())
             
-            for i in xrange(self.notebook.get_n_pages()):
-                page = self.notebook.get_nth_page(i)
-                page.toolbar.profile_entry.update()
+            self.scan_interface.toolbar.profile_entry.update()
         else:
             target = self.directions['Choose'].target_entry.get_text()
             cmd = command % target
             
-            current_page = self.notebook.get_nth_page(self.notebook.get_current_page())
-            if current_page == None:
-                current_page = self.notebook.add_scan_page(target)
+            self.scan_interface.execute_command(cmd)
 
-            current_page.execute_command(cmd)
-
-            current_page.toolbar.target_entry.selected_target = self.\
+            self.scan_interface.toolbar.target_entry.selected_target = self.\
                                 directions['Choose'].target_entry.get_text()
-            current_page.command_toolbar.command_entry.command = cmd
-            current_page.command_toolbar.set_command(cmd)
+            self.scan_interface.command_toolbar.command_entry.command = cmd
+            self.scan_interface.command_toolbar.set_command(cmd)
         
         self.close_wizard()
 
@@ -351,23 +336,17 @@ class ProfilePage(HIGVBox):
         self.set_spacing(12)
         self.prof = False
         
-        self.description = HIGEntryLabel(_("""Please, enter the profile name, \
-and optionally, enter a hint, description and annotation for this \
-new profile"""))
+        self.description = HIGEntryLabel(_("""Please enter a name \
+and optionally a description for this new profile"""))
         self.profile_label = HIGEntryLabel(_("Profile name"))
-        self.hint_label = HIGEntryLabel(_("Hint"))
         self.description_label = HIGEntryLabel(_("Description"))
-        self.annotation_label = HIGEntryLabel(_("Annotation"))
         
         self.profile_entry = gtk.Entry()
-        self.hint_entry = gtk.Entry()
         self.description_scroll = HIGScrolledWindow()
+        self.description_scroll.set_border_width(0)
         self.description_text = HIGTextView()
-        self.annotation_scroll = HIGScrolledWindow()
-        self.annotation_text = HIGTextView()
         
         self.description_scroll.add(self.description_text)
-        self.annotation_scroll.add(self.annotation_text)
         
         table = HIGTable()
         self.bar = ForwardBar()
@@ -376,17 +355,11 @@ new profile"""))
         self._pack_expand_fill(table)
         self._pack_noexpand_nofill(self.bar)
         
-        table.attach(self.profile_label,0,1,0,1,xoptions=0)
-        table.attach(self.profile_entry,1,2,0,1)
+        table.attach(self.profile_label,0,1,0,1,xoptions=0,yoptions=0)
+        table.attach(self.profile_entry,1,2,0,1,yoptions=0)
         
-        table.attach(self.hint_label,0,1,1,2,xoptions=0)
-        table.attach(self.hint_entry,1,2,1,2)
-        
-        table.attach(self.description_label,0,1,2,3,xoptions=0)
-        table.attach(self.description_scroll,1,2,2,3)
-        
-        table.attach(self.annotation_label,0,1,3,4,xoptions=0)
-        table.attach(self.annotation_scroll,1,2,3,4)
+        table.attach(self.description_label,0,1,1,2,xoptions=0)
+        table.attach(self.description_scroll,1,2,1,2)
 
 class StartPage(HIGVBox):
     def __init__(self):
